#include <math.h>
#include <ArpCore/ArpDebug.h>
#include <ArpCore/StlVector.h>
#include <be/support/Errors.h>
#include "ArpMath/ArpDefs.h"
#include "GlPublic/GlGrowthPattern.h"

/*******************************************************
 * GL-GROWTH-PATTERN
 *******************************************************/
class _CNext
{
public:
	bool		on;
	uint32		dirLT[GL_GROWTH_LIMIT],	dirT[GL_GROWTH_LIMIT],	dirRT[GL_GROWTH_LIMIT],
				dirL[GL_GROWTH_LIMIT],							dirR[GL_GROWTH_LIMIT],
				dirLB[GL_GROWTH_LIMIT],	dirB[GL_GROWTH_LIMIT],	dirRB[GL_GROWTH_LIMIT];
	
	_CNext()
	{
		on = false;
		for (int32 k = 0; k < GL_GROWTH_LIMIT; k++) {
			dirLT[k] = dirT[k] = dirRT[k] = 0;
			dirL[k] = dirR[k] = 0;
			dirLB[k] = dirB[k] = dirRB[k] = 0;
		}
	}

	void				MakeDirs()
	{
		/* I must have something in each directiont to be valid.
		 */
		if (MakeDir(dirLT) != B_OK) return;
		if (MakeDir(dirT) != B_OK) return;
		if (MakeDir(dirRT) != B_OK) return;
		if (MakeDir(dirL) != B_OK) return;
		if (MakeDir(dirR) != B_OK) return;
		if (MakeDir(dirLB) != B_OK) return;
		if (MakeDir(dirB) != B_OK) return;
		if (MakeDir(dirRB) != B_OK) return;
		on = true;
	}

	_gl_dir_pattern*	NewPattern()
	{
		if (!on) return 0;
		_gl_dir_pattern*	pattern = new _gl_dir_pattern();
		if (!pattern) return 0;
		AddDir(dirLT,	pattern->dir[0]);
		AddDir(dirT,	pattern->dir[1]);
		AddDir(dirRT,	pattern->dir[2]);
		AddDir(dirL,	pattern->dir[3]);
		AddDir(dirR,	pattern->dir[4]);
		AddDir(dirLB,	pattern->dir[5]);
		AddDir(dirB,	pattern->dir[6]);
		AddDir(dirRB,	pattern->dir[7]);
		return pattern;
	}
		
private:
	status_t			MakeDir(uint32* dest)
	{
		uint32				k, total = 0;
		for (k = 0; k < GL_GROWTH_LIMIT; k++) total += dest[k];
		if (total < 1) return B_ERROR;

		vector<uint8>		vec;
		uint32				i = 0;
		for (k = 0; k < GL_GROWTH_LIMIT; k++) {
			if (dest[k] > 0) {
				uint32		num = uint32(ceil((dest[k] / float(total)) * GL_GROWTH_LIMIT));
				for (uint32 j = i; i < j + num; i++)
					vec.push_back(uint8(k));
			}
		}
		if (vec.size() < GL_GROWTH_LIMIT) {
			printf("FAILED total %ld vec size %ld\n", total, vec.size());
			if (total == 1) {
				while (vec.size() < GL_GROWTH_LIMIT) vec.push_back(vec[0]);
			}
		}
		ArpASSERT(vec.size() >= GL_GROWTH_LIMIT);
		while (vec.size() > GL_GROWTH_LIMIT) {
			uint32		diff = uint32(vec.size() - GL_GROWTH_LIMIT);
			uint32		rem = GL_GROWTH_LIMIT / diff;
			for (k = 0; k < diff; k++) {
				if (vec.size() > GL_GROWTH_LIMIT) {
					uint32	e = k * rem;
					if (e < vec.size()) vec.erase(vec.begin() + e);
				}
			}
		}
		ArpASSERT(vec.size() == GL_GROWTH_LIMIT);
#if 0
		printf("**LEFT TOP NORM'D (total %ld)\n", total);
		for (k = 0; k < vec.size(); k++) {
			printf("%ld: %d\n", k, vec[k]);
		}
#endif

		for (k = 0; k < GL_GROWTH_LIMIT; k++) dest[k] = vec[k];

		return B_OK;
	}

	void			AddDir(uint32* src, uint8* dest)
	{
		for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++)
			dest[k] = ARP_CLIP_255(src[k]);
	}

public:
	void			Print(uint8 dir)
	{
#if 0
		printf("DIR %d\n", dir);
		if (dir == 7)
		for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++) {
			
		}
#endif
	}
};

static bool invalid_dir(_gl_dir_pattern* pattern[GL_GROWTH_LIMIT], uint8* dir)
{
	for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++) {
		if (!pattern[dir[k]]) return true;
	}
	return false;
}

static bool invalid_pattern(_gl_dir_pattern* pattern[GL_GROWTH_LIMIT], uint32 k)
{
	if (invalid_dir(pattern, pattern[k]->dir[0])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[1])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[2])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[3])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[4])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[5])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[6])) return true;
	if (invalid_dir(pattern, pattern[k]->dir[7])) return true;
	return false;
}

static bool invalid_neighbors(_gl_dir_pattern* pattern[GL_GROWTH_LIMIT])
{
	bool		ans = false;
	for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++) {
		if (pattern[k]) {
			if (invalid_pattern(pattern, k)) {
				delete pattern[k];
				pattern[k] = 0;
				ans = true;
			}
		}
	}
	return ans;
}

/*******************************************************
 * GL-GROWTH-PATTERN
 *******************************************************/
GlGrowthPattern::GlGrowthPattern(const uint8* bytes, int32 w, int32 h)
		: mStatus(B_ERROR)
{
	for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++) mPattern[k] = 0;
	ArpASSERT(bytes);
	int32		x, y, pix;
	_CNext*		next = new _CNext[GL_GROWTH_LIMIT];
	if (!next) return;

	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			pix = ARP_PIXEL(x, y, w);
			uint8			ni = bytes[pix];
			next[ni].dirLT[bytes[ARP_PIXEL_SE(x - 1, y - 1, w, h)]]++;
			next[ni].dirT[bytes[ARP_PIXEL_SE(x, y - 1, w, h)]]++;
			next[ni].dirRT[bytes[ARP_PIXEL_SE(x + 1, y - 1, w, h)]]++;

			next[ni].dirL[bytes[ARP_PIXEL_SE(x - 1, y, w, h)]]++;
			next[ni].dirR[bytes[ARP_PIXEL_SE(x + 1, y, w, h)]]++;

			next[ni].dirLB[bytes[ARP_PIXEL_SE(x - 1, y + 1, w, h)]]++;
			next[ni].dirB[bytes[ARP_PIXEL_SE(x, y + 1, w, h)]]++;
			next[ni].dirRB[bytes[ARP_PIXEL_SE(x + 1, y + 1, w, h)]]++;

#if 0
			if ( (col = x - 1) >= 0 && col < w && (row = y - 1) >= 0 && row < h)
				next[ni].dirLT[bytes[ARP_PIXEL(col, row, w)]]++;
			if ( (col = x) >= 0 && col < w && (row = y - 1) >= 0 && row < h)
				next[ni].dirT[bytes[ARP_PIXEL(col, row, w)]]++;
			if ( (col = x + 1) >= 0 && col < w && (row = y - 1) >= 0 && row < h)
				next[ni].dirRT[bytes[ARP_PIXEL(col, row, w)]]++;

			if ( (col = x - 1) >= 0 && col < w && (row = y) >= 0 && row < h)
				next[ni].dirL[bytes[ARP_PIXEL(col, row, w)]]++;
			if ( (col = x + 1) >= 0 && col < w && (row = y) >= 0 && row < h)
				next[ni].dirR[bytes[ARP_PIXEL(col, row, w)]]++;

			if ( (col = x - 1) >= 0 && col < w && (row = y + 1) >= 0 && row < h)
				next[ni].dirLB[bytes[ARP_PIXEL(col, row, w)]]++;
			if ( (col = x) >= 0 && col < w && (row = y + 1) >= 0 && row < h)
				next[ni].dirB[bytes[ARP_PIXEL(col, row, w)]]++;
			if ( (col = x + 1) >= 0 && col < w && (row = y + 1) >= 0 && row < h)
				next[ni].dirRB[bytes[ARP_PIXEL(col, row, w)]]++;
#endif
		}
	}
	/* Build up our neighbor information.  Anyone who points
	 * to an invalid neighbor is also invalid.
	 */
	for (pix = 0; pix < GL_GROWTH_LIMIT; pix++) next[pix].MakeDirs();
//next[238].Print();
	/* Wipe out invalid neighbors.
	 */
//	while (invalid_neighbors(mPattern)) ;
	/* Build my actual pattern data.
	 */
	for (pix = 0; pix < GL_GROWTH_LIMIT; pix++) {
		mPattern[pix] = next[pix].NewPattern();
#if 0
		if (next[pix].on == true) printf("%ld: on\n", pix);
		else printf("%ld: OFF\n", pix);
		if (next[pix].on == true) ArpASSERT(mPattern[pix]);
#endif
	}
	delete next;
	mStatus = B_OK;
}

GlGrowthPattern::~GlGrowthPattern()
{
	for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++) delete mPattern[k];
}

status_t GlGrowthPattern::InitCheck() const
{
	return mStatus;
}

uint8 GlGrowthPattern::InitialValue() const
{
	/* FIX:  Should be random.
	 */
	for (uint32 k = 0; k < GL_GROWTH_LIMIT; k++) {
		if (mPattern[k]) return uint8(k);
	}
	return 0;
}

uint8 GlGrowthPattern::NextValue(uint8 prev, uint8 dir, uint8 random) const
{
	if (!mPattern[prev]) {
		printf("Failing on %d\n", prev);
	}
	ArpVALIDATE(mPattern[prev], return 0);
	ArpVALIDATE(dir < 8, return 0);
uint8	next = mPattern[prev]->dir[dir][random];
if (!mPattern[next]) {
	printf("Returning an invalid next (%d) from prev %d dir %d\n", next, prev, dir);
	ArpASSERT(false);
}
	return mPattern[prev]->dir[dir][random];
}

// #pragma mark -

/*******************************************************
 * Predefined patterns
 *******************************************************/
const int32	gFernMossW = 98;
const int32 gFernMossH = 99;
const uint8 gFernMossData[] = {
	156, 167, 168, 141, 142, 154, 194, 183, 157, 154, 158, 173, 174, 168, 175, 177, 191, 185, 177, 175, 179, 175, 187, 169, 177, 187, 181, 179, 163, 135, 191, 184, 210, 171, 175, 209, 189, 201, 181, 160, 142, 159, 173, 169, 182, 191, 155, 201, 185, 170, 207, 187, 171, 200, 189, 197, 173, 146, 143, 164, 180, 199, 213, 181, 157, 167, 149, 122, 110, 126, 163, 184, 154, 144, 164, 160, 161, 185, 95, 134, 147, 148, 167, 152, 132, 141, 147, 147, 136, 130, 120, 130, 127, 146, 144, 148, 149, 102, 164, 172, 164, 168, 172, 174, 174, 169, 162, 149, 143, 154, 156, 172, 175, 185, 182, 191, 175, 175, 172, 177, 184, 164, 159, 176, 198, 164, 147, 157, 214, 208, 227, 191, 173, 195, 165, 189, 155, 148, 166, 174, 173, 171, 161, 179, 174, 170, 165, 173, 194, 178, 141, 118, 170, 187, 187, 170, 156, 169, 188, 185, 192, 164, 146, 154, 149, 124, 118, 131, 161, 167, 154, 161, 169, 128, 108, 161, 174, 158, 190, 204, 195, 182, 159, 125, 123, 139, 164, 150, 140, 128, 144, 154, 117, 139, 140, 128, 174, 151, 147, 158, 176, 196, 196, 189, 176, 167, 147, 148, 168, 185, 182, 187, 185, 189, 170, 168, 170, 168, 217, 204, 140, 161, 204, 173, 158, 225, 182, 182, 224, 191, 164, 187, 173, 196, 134, 150, 177, 220, 209, 177, 162, 144, 196, 162, 177, 195, 185, 187, 141, 65, 138, 153, 160, 169, 178, 189, 187, 185, 189, 156, 140, 155, 149, 131, 135, 148, 187, 178, 166, 173, 158, 93, 72, 124, 139, 111, 146, 178, 160, 158, 142, 94, 102, 135, 159, 155, 127, 128, 147, 168, 104, 91, 129, 183, 182, 185, 187, 194, 159, 186, 159, 172, 174, 174, 155, 143, 161, 165, 185, 172, 195, 194, 168, 168, 184, 175, 181, 189, 194, 170, 184, 187, 199, 189, 199, 201, 208, 220, 201, 177, 169, 201, 164, 194, 177, 191, 147, 95, 147, 189, 171, 122, 186, 179, 142, 155, 162, 145, 80, 180, 206, 164, 158, 182, 182, 177, 211, 133, 124, 176, 177, 157, 131, 114, 141, 130, 154, 122, 92, 144, 175, 111, 142, 83, 88, 151, 162, 158, 135, 91, 113, 156, 161, 148, 138, 117, 134, 191, 143, 186, 169, 186, 177, 170, 159, 177, 163, 199, 140, 166, 193, 193, 181, 158, 154, 160, 194, 177, 199, 202, 181, 170, 175, 137, 204, 173, 161, 179, 181, 165, 179, 204, 181, 194, 208, 221, 201, 168, 163, 165, 169, 187, 158, 178, 180, 153, 198, 194, 133, 166, 226, 163, 147, 199, 169, 129, 134, 184, 198, 202, 208, 168, 114, 97, 223, 145, 138, 187, 181, 165, 156, 149, 152, 151, 148, 141, 134, 172, 161, 114, 110, 72, 95, 139, 145, 139, 138, 123, 127, 151, 155, 125, 84, 92, 99, 101, 148, 205, 177, 176, 181, 146, 148, 181, 174, 187, 148, 169, 214, 210, 187, 165, 174, 191, 174, 165, 192, 212, 187, 175, 165, 135, 205, 154, 138, 161, 185, 165, 168, 219, 183, 180, 202, 218, 196, 163, 157, 165, 204, 194, 157, 182, 214, 199, 221, 208, 177, 227, 189, 65, 72, 182, 190, 181, 194, 174, 159, 174, 205, 177, 128, 134, 216, 158, 154, 192, 192, 177, 173, 168, 135, 145, 133, 141, 172, 154, 136, 119, 113, 100, 119, 147, 141, 147, 175, 163, 136, 132, 159, 129, 77, 117, 157, 133, 139, 181, 149, 153, 165, 129, 147, 199, 160, 168, 173, 185, 201, 194, 169, 158, 184, 198, 156, 151, 191, 214, 201, 189, 165, 119, 228, 178, 149, 161, 177, 181, 187, 209, 186, 193, 194, 187, 178, 187, 201, 209, 194, 201, 162, 194, 201, 188, 223, 201, 103, 181, 211, 112, 135, 195, 130, 111, 175, 162, 142, 155, 192, 185, 166, 204, 214, 158, 163, 182, 175, 167, 175, 172, 141, 150, 138, 139, 178, 141, 90, 135, 148, 153, 182, 171, 127, 142, 185, 184, 161, 117, 150, 166, 89, 106, 193, 196, 124, 174, 131, 126, 151, 141, 178, 211, 173, 148, 187, 185, 168, 156, 154, 152, 159, 169, 166, 160, 199, 211, 189, 177, 163, 119, 202, 204, 182, 151, 154, 194, 204, 194, 194, 199, 192, 187, 191, 196, 202, 212, 134, 183, 168, 172, 167, 148, 185, 182, 127, 176, 205, 165, 188, 188, 115, 142, 84, 159, 188, 178, 188, 174, 164, 168, 192, 153, 149, 168, 156, 148, 154, 147, 155, 176, 159, 164, 186, 121, 104, 166, 159, 163, 184, 157, 99, 122, 151, 165, 175, 113, 135, 157, 61, 51, 143, 179, 144, 157, 144, 125, 140, 164, 197, 218, 157, 154, 192, 187, 177, 175, 177, 172, 164, 149, 189, 183, 206, 209, 175, 160, 156, 129, 175, 201, 195, 148, 144, 198, 200, 176, 174, 200, 204, 200, 207, 193, 184, 161, 123, 174, 181, 193, 168, 144, 188, 180, 227, 191, 188, 172, 171, 174, 151, 222, 101, 188, 207, 182, 178, 174, 173, 195, 195, 140, 137, 163, 151, 147, 156, 123, 163, 170, 177, 173, 155, 126, 125, 191, 140, 163, 189, 142, 95, 129, 162, 144, 139, 84, 109, 131, 54, 47, 107, 164, 152, 160, 113, 121, 157, 188, 215, 200, 149, 177, 214, 190, 192, 178, 174, 173, 182, 185, 190, 184, 202, 204, 158, 151, 169, 159, 157, 192, 194, 177, 168, 188, 204, 197, 171, 191, 210, 201, 204, 197, 177, 151, 141, 198, 186, 194, 195, 174, 189, 149, 184, 150, 216, 244, 209, 189, 147, 144, 165, 187, 174, 172, 176, 171, 171, 207, 194, 139, 136, 163, 163, 176, 155, 124, 152, 160, 187, 181, 153, 139, 171, 184, 160, 177, 189, 159, 112, 151, 167, 129, 88, 105, 116, 109, 120, 121, 156, 171, 145, 153, 103, 115, 169, 204, 204, 182, 141, 210, 195, 192, 172, 141, 123, 142, 182, 199, 174, 177, 206, 194, 148, 158, 198, 208, 155, 177, 198, 209, 198, 187, 206, 225, 191, 199, 196, 183, 187, 206, 201, 172, 187, 190, 174, 178, 203, 181, 163, 77, 188, 145, 202, 197, 152, 176, 151, 144, 178, 151, 140, 191, 214, 146, 85, 111, 186, 150, 142, 163, 181, 195, 179, 137, 172, 155, 193, 211, 167, 191, 228, 199, 163, 187, 201, 141, 115, 152, 158, 97, 77, 133, 139, 111, 153, 204, 189, 150, 160, 149, 112, 135, 198, 183, 182, 158, 175, 182, 183, 199, 188, 139, 104, 141, 190, 192, 156, 161, 155, 154, 148, 171, 178, 177, 139, 153, 175, 205, 211, 187, 175, 192, 182, 190, 204, 182, 166, 194, 207, 170, 189, 126, 120, 224, 227, 134, 104, 152, 186, 164, 170, 191, 205, 181, 142, 94, 183, 195, 172, 164, 171, 149, 138, 157, 196, 140, 130, 157, 165, 167, 185, 157, 182, 105, 134, 201, 132, 110, 158, 199, 185, 195, 188, 212, 165, 112, 161, 132, 104, 120, 125, 145, 160, 182, 161, 120, 123, 199, 156, 163, 165, 184, 190, 185, 173, 173, 183, 168, 157, 136, 119, 170, 207, 184, 168, 174, 188, 179, 184, 205, 200, 187, 179, 176, 181, 178, 177, 189, 207, 223, 219, 206, 200, 176, 154, 188, 224, 212, 224, 179, 170, 191, 174, 149, 146, 160, 196, 192, 189, 194, 188, 185, 167, 152, 178, 204, 205, 186, 173, 130, 136, 198, 183, 136, 140, 169, 165, 180, 187, 172, 181, 127, 159, 196, 127, 91, 146, 183, 191, 180, 144, 158, 134, 130, 178, 142, 96, 112, 135, 161, 168, 165, 156, 156, 136, 141, 128, 160, 147, 155, 164, 163, 161, 173, 156, 137, 149, 168, 168, 192, 195, 164, 178, 203, 211, 209, 203, 198, 184, 158, 181, 199, 188, 165, 153, 184, 213, 206, 213, 213, 204, 189, 173, 195, 222, 231, 204, 214, 196, 155, 140, 171, 187, 176, 211, 219, 219, 192, 184, 169, 171, 184, 159, 195, 196, 182, 188, 148, 136, 176, 186, 159, 154, 160, 154, 178, 192, 178, 163, 128, 180, 188, 123, 106, 161, 185, 208, 158, 97, 115, 121, 139, 193, 157, 98, 110, 148, 182, 158, 128, 129, 158, 174, 81, 105, 160, 175, 154, 123, 98, 135, 190, 157, 136, 156, 192, 200, 200, 190, 164, 162, 187, 207, 215, 205, 194, 176, 151, 164, 203, 198, 167, 160, 181, 187, 155, 184, 207, 209, 205, 202, 185, 174, 190, 153, 178, 190, 162, 160, 179, 199, 197, 190, 206, 202, 192, 178, 161, 167, 181, 176, 161, 133, 147, 193, 180, 140, 137, 196, 167, 177, 187, 163, 145, 169, 170, 185, 145, 163, 160, 102, 122, 180, 188, 204, 167, 103, 107, 115, 138, 200, 147, 105, 113, 138, 167, 152, 109, 124, 176, 165, 80, 124, 147, 184, 176, 137, 134, 159, 199, 190, 162, 186, 196, 180, 169, 180, 181, 159, 181, 198, 208, 212, 216, 203, 185, 165, 199, 204, 167, 170, 180, 162, 117, 155, 183, 180, 188, 200, 151, 117, 154, 161, 177, 190, 191, 172, 159, 177, 188, 137, 149, 164, 175, 182, 181, 159, 152, 186, 171, 136, 161, 204, 188, 141, 142, 201, 176, 177, 191, 154, 148, 180, 177, 212, 162, 156, 147, 98, 133, 200, 167, 169, 173, 141, 147, 137, 152, 196, 152, 127, 96, 111, 151, 157, 139, 158, 200, 119, 81, 146, 122, 155, 175, 187, 190, 179, 184, 223, 203, 193, 163, 123, 142, 201, 217, 181, 197, 194, 178, 177, 179, 178, 164, 202, 191, 164, 144, 162, 190, 177, 143, 168, 176, 136, 138, 168, 125, 97, 158, 241, 214, 197, 200, 172, 137, 153, 190, 159, 154, 161, 170, 181, 164, 133, 112, 175, 204, 195, 195, 197, 174, 162, 195, 190, 160, 169, 184, 161, 158, 183, 187, 176, 113, 115, 137, 120, 166, 210, 190, 122, 167, 167, 182, 144, 149, 210, 167, 182, 147, 143, 159, 161, 154, 155, 175, 71, 97, 147, 114, 153, 179, 201, 193, 173, 173, 208, 200, 193, 152, 107, 151, 206, 211, 221, 200, 184, 152, 143, 156, 177, 171, 189, 140, 110, 134, 164, 205, 196, 172, 202, 187, 120, 124, 195, 140, 117, 197, 243, 231, 208, 194, 163, 155, 167, 193, 203, 187, 174, 165, 174, 156, 141, 117, 184, 209, 199, 195, 194, 166, 165, 196, 193, 155, 163, 175, 164, 171, 183, 181, 97, 60, 110, 161, 162, 170, 197, 173, 116, 171, 182, 179, 129, 138, 198, 181, 182, 182, 177, 159, 158, 158, 144, 139, 84, 145, 148, 148, 179, 204, 212, 187, 186, 191, 169, 182, 191, 157, 128, 167, 216, 179, 206, 205, 172, 145, 146, 186, 224, 234, 146, 106, 76, 123, 188, 209, 206, 172, 225, 206, 136, 153, 221, 185, 145, 228, 208, 209, 208, 191, 177, 185, 187, 188, 190, 191, 176, 169, 176, 177, 179, 189, 207, 184, 161, 161, 196, 176, 149, 168, 194, 165, 161, 174, 157, 157, 178, 171, 93, 60, 134, 207, 173, 142, 146, 120, 133, 184, 178, 165, 112, 113, 186, 157, 121, 153, 165, 163, 161, 158, 158, 151, 135, 192, 152, 181, 177, 173, 194, 188, 180, 209, 174, 172, 184, 220, 188, 153, 179, 235, 207, 154, 167, 201, 204, 174, 171, 215, 119, 101, 100, 161, 205, 191, 167, 221, 199, 237, 220, 146, 159, 220, 205, 158, 211, 117, 141, 220, 189, 110, 141, 204, 164, 162, 154, 178, 202, 199, 185, 206, 198, 193, 97, 129, 211, 190, 166, 185, 206, 163, 145, 178, 180, 155, 147, 147, 137, 146, 117, 140, 196, 156, 104, 125, 148, 158, 165, 155, 144, 175, 205, 171, 185, 117, 157, 199, 137, 168, 198, 154, 198, 202, 182, 177, 179, 182, 212, 205, 167, 174, 186, 177, 164, 166, 175, 180, 176, 200, 196, 191, 184, 194, 206, 202, 198, 188, 164, 146, 121, 157, 207, 196, 178, 207, 185, 185, 182, 142, 143, 195, 211, 180, 197, 117, 134, 204, 206, 172, 190, 221, 182, 172, 169, 169, 176, 176, 181, 167, 159, 228, 188, 179, 193, 141, 153, 173, 194, 179, 168, 179, 181, 159, 143, 178, 142, 173, 171, 170, 153, 110, 97, 130, 171, 174, 184, 148, 133, 161, 188, 154, 184, 146, 177, 191, 152, 157, 189, 171, 181, 179, 168, 152, 161, 183, 209, 184, 139, 120, 200, 191, 160, 134, 167, 198, 203, 159, 198, 196, 187, 180, 207, 232, 219, 165, 171, 179, 165, 172, 220, 205, 179, 163, 172, 187, 177, 163, 171, 184, 207, 212, 195, 137, 141, 200, 207, 206, 207, 196, 215, 169, 181, 168, 142, 169, 188, 119, 118, 231, 224, 200, 200, 171, 193, 173, 182, 182, 180, 183, 168, 147, 150, 191, 130, 135, 178, 178, 126, 113, 142, 154, 197, 181, 175, 162, 129, 161, 189, 158, 174, 161, 159, 156, 151, 129, 144, 169, 166, 155, 143, 137, 182, 177, 190, 163, 127, 127, 222, 184, 148, 150, 173, 207, 204, 177, 204, 186, 172, 179, 209, 224, 195, 159, 129, 184, 198, 198, 223, 211, 163, 141, 196, 187, 196, 200, 195, 178, 183, 212, 183, 155, 168, 180, 182, 211, 197, 161, 215, 169, 187, 177, 129, 185, 201, 108, 128, 190, 186, 207, 222, 199, 207, 170, 194, 180, 175, 185, 161, 138, 158, 178, 150, 115, 142, 163, 139, 151, 163, 125, 183, 154, 158, 145, 134, 173, 205, 176, 142, 155, 142, 134, 131, 119, 124, 164, 164, 157, 141, 119, 221, 199, 185, 156, 154, 174, 218, 163, 145, 168, 197, 195, 187, 213, 219, 169, 154, 190, 222, 211, 177, 178, 132, 195, 215, 190, 204, 206, 184, 176, 187, 177, 197, 226, 214, 178, 173, 204, 138, 154, 174, 190, 179, 206, 221, 179, 187, 158, 193, 178, 135, 188, 209, 112, 193, 188, 161, 204, 185, 151, 168, 136, 182, 181, 175, 183, 176, 166, 170, 172, 179, 142, 157, 178, 135, 135, 137, 76, 166, 144, 137, 147, 142, 176, 221, 206, 167, 164, 145, 145, 148, 137, 141, 182, 153, 143, 125, 135, 213, 182, 165, 155, 169, 189, 187, 155, 151, 191, 196, 163, 173, 213, 236, 175, 161, 207, 218, 184, 172, 199, 165, 213, 209, 171, 184, 184, 199, 219, 162, 150, 177, 211, 217, 180, 164, 190, 154, 187, 224, 204, 175, 199, 205, 172, 167, 152, 182, 181, 154, 181, 198, 137, 181, 179, 174, 208, 138, 71, 140, 160, 189, 186, 187, 197, 188, 189, 174, 170, 155, 170, 190, 171, 130, 122, 121, 107, 176, 141, 138, 138, 142, 178, 204, 191, 171, 160, 154, 160, 156, 154, 164, 174, 129, 140, 139, 140, 163, 147, 163, 180, 152, 143, 178, 163, 181, 201, 206, 170, 182, 201, 197, 193, 188, 193, 186, 172, 186, 200, 211, 211, 190, 167, 180, 187, 184, 200, 173, 153, 169, 204, 212, 204, 197, 197, 184, 200, 238, 224, 177, 193, 190, 160, 162, 181, 194, 210, 191, 179, 175, 168, 165, 175, 195, 198, 101, 83, 174, 198, 163, 178, 200, 195, 181, 171, 175, 164, 137, 170, 165, 140, 144, 150, 160, 160, 185, 146, 138, 144, 139, 159, 188, 171, 189, 134, 148, 167, 146, 151, 170, 137, 119, 148, 150, 151, 143, 123, 188, 218, 158, 110, 162, 200, 211, 200, 200, 208, 196, 172, 179, 193, 203, 167, 153, 165, 179, 197, 194, 199, 172, 173, 209, 185, 143, 152, 207, 200, 193, 211, 229, 228, 217, 222, 168, 183, 215, 222, 197, 193, 213, 163, 182, 209, 228, 237, 225, 190, 163, 182, 175, 208, 215, 189, 109, 122, 217, 172, 139, 170, 205, 194, 178, 148, 152, 155, 172, 176, 110, 92, 179, 212, 172, 172, 196, 144, 132, 145, 149, 159, 175, 164, 208, 127, 154, 178, 138, 160, 180, 112, 139, 168, 177, 165, 187, 140, 154, 173, 159, 201, 186, 192, 203, 179, 159, 182, 182, 179, 213, 191, 168, 181, 177, 179, 200, 233, 197, 222, 165, 167, 216, 153, 97, 160, 213, 97, 119, 190, 178, 187, 229, 195, 158, 149, 240, 224, 176, 193, 164, 201, 212, 195, 183, 183, 198, 205, 198, 192, 188, 182, 205, 203, 145, 151, 193, 212, 178, 184, 193, 201, 183, 154, 145, 177, 177, 182, 149, 140, 165, 172, 157, 160, 189, 175, 152, 144, 142, 158, 167, 175, 198, 118, 61, 94, 147, 190, 198, 191, 147, 150, 137, 113, 184, 144, 163, 174, 179, 173, 158, 164, 177, 186, 188, 176, 184, 191, 196, 182, 190, 199, 190, 170, 160, 182, 200, 197, 178, 181, 202, 161, 133, 195, 198, 127, 145, 199, 200, 224, 243, 238, 188, 138, 204, 194, 194, 225, 172, 160, 171, 185, 182, 194, 195, 184, 169, 153, 188, 185, 204, 198, 152, 157, 188, 188, 207, 183, 169, 172, 165, 172, 176, 160, 191, 187, 149, 142, 160, 172, 148, 152, 186, 169, 152, 140, 141, 156, 156, 156, 152, 91, 66, 93, 166, 188, 167, 117, 139, 159, 148, 126, 184, 168, 156, 193, 191, 142, 165, 165, 190, 210, 208, 178, 189, 206, 194, 196, 204, 203, 172, 150, 153, 159, 188, 188, 190, 192, 197, 177, 183, 219, 195, 187, 190, 204, 212, 221, 235, 234, 230, 174, 184, 164, 184, 212, 167, 167, 178, 184, 198, 211, 195, 184, 185, 181, 188, 172, 181, 185, 172, 179, 195, 186, 200, 195, 173, 157, 171, 189, 173, 158, 180, 190, 152, 133, 148, 153, 132, 152, 165, 154, 149, 152, 155, 155, 149, 144, 169, 159, 150, 138, 144, 161, 156, 157, 166, 166, 163, 140, 198, 191, 164, 188, 205, 126, 182, 168, 189, 228, 213, 170, 163, 200, 194, 198, 194, 175, 142, 133, 146, 185, 205, 188, 195, 204, 192, 188, 224, 221, 185, 214, 196, 179, 200, 220, 194, 200, 234, 231, 207, 170, 156, 162, 170, 196, 193, 204, 201, 197, 179, 181, 196, 209, 178, 157, 169, 186, 175, 198, 206, 178, 184, 187, 196, 211, 195, 164, 170, 160, 174, 172, 140, 130, 151, 145, 135, 165, 156, 165, 166, 166, 169, 165, 158, 138, 143, 148, 136, 118, 130, 147, 142, 124, 157, 173, 160, 138, 201, 205, 168, 189, 219, 161, 196, 182, 193, 217, 187, 153, 157, 203, 178, 193, 185, 155, 129, 146, 178, 207, 221, 190, 188, 192, 190, 224, 222, 197, 161, 181, 169, 149, 195, 217, 212, 183, 177, 212, 178, 174, 189, 173, 172, 205, 198, 196, 195, 184, 177, 182, 190, 215, 188, 164, 176, 182, 176, 188, 195, 174, 177, 190, 212, 224, 183, 119, 126, 188, 191, 177, 144, 143, 165, 154, 142, 172, 151, 167, 172, 162, 172, 175, 159, 124, 84, 89, 116, 153, 193, 217, 159, 66, 135, 145, 141, 128, 195, 189, 188, 214, 226, 195, 193, 191, 205, 222, 198, 167, 180, 216, 159, 179, 185, 180, 173, 178, 187, 210, 207, 205, 184, 165, 191, 218, 202, 168, 190, 184, 172, 160, 177, 204, 202, 164, 99, 167, 125, 168, 246, 213, 190, 178, 186, 199, 205, 207, 205, 200, 198, 198, 205, 182, 188, 189, 162, 168, 176, 158, 196, 188, 202, 194, 131, 95, 134, 216, 200, 186, 147, 155, 190, 171, 149, 172, 149, 169, 172, 155, 157, 169, 163, 127, 61, 151, 207, 182, 156, 168, 154, 130, 157, 157, 164, 145, 184, 169, 203, 227, 211, 190, 167, 203, 228, 220, 204, 183, 193, 202, 168, 178, 211, 192, 197, 204, 205, 192, 171, 224, 180, 144, 192, 209, 161, 165, 224, 190, 199, 199, 158, 179, 191, 151, 114, 189, 130, 168, 246, 203, 171, 181, 208, 198, 204, 207, 218, 220, 209, 203, 195, 188, 199, 198, 145, 144, 178, 181, 189, 209, 197, 148, 126, 141, 178, 190, 179, 149, 123, 154, 194, 193, 164, 181, 152, 174, 175, 152, 155, 178, 165, 130, 54, 163, 214, 140, 65, 68, 133, 164, 175, 194, 181, 158, 186, 169, 221, 221, 177, 164, 136, 184, 215, 206, 178, 164, 167, 166, 176, 202, 198, 192, 191, 211, 205, 191, 155, 224, 187, 132, 190, 184, 150, 161, 221, 163, 197, 216, 149, 170, 217, 188, 156, 245, 185, 185, 200, 143, 167, 219, 212, 198, 177, 178, 185, 193, 205, 207, 171, 176, 205, 194, 140, 144, 185, 199, 182, 221, 198, 137, 151, 222, 219, 149, 125, 103, 80, 123, 207, 195, 164, 187, 174, 188, 183, 153, 157, 187, 183, 151, 80, 136, 159, 129, 131, 189, 181, 138, 152, 175, 179, 160, 216, 165, 177, 181, 181, 195, 175, 179, 190, 205, 218, 207, 202, 207, 173, 222, 207, 150, 144, 214, 207, 143, 173, 214, 185, 176, 191, 168, 171, 226, 228, 200, 185, 200, 184, 170, 191, 222, 203, 215, 208, 184, 184, 191, 186, 161, 154, 172, 188, 193, 181, 172, 194, 206, 165, 188, 188, 164, 144, 169, 196, 208, 200, 224, 144, 81, 128, 198, 172, 124, 115, 132, 171, 184, 187, 169, 161, 149, 164, 174, 164, 156, 172, 193, 177, 136, 62, 60, 140, 137, 160, 136, 193, 169, 152, 186, 170, 158, 195, 158, 172, 188, 190, 200, 195, 178, 176, 202, 205, 204, 200, 200, 167, 196, 184, 143, 154, 202, 208, 160, 161, 204, 195, 160, 184, 185, 174, 222, 217, 214, 197, 167, 183, 202, 198, 200, 208, 204, 208, 200, 216, 216, 204, 186, 178, 182, 186, 196, 202, 199, 204, 202, 161, 172, 185, 167, 169, 182, 178, 171, 181, 181, 157, 128, 157, 181, 154, 142, 171, 176, 178, 167, 149, 154, 144, 149, 164, 157, 157, 158, 171, 176, 152, 128, 100, 101, 161, 171, 182, 157, 183, 160, 119, 162, 175, 168, 154, 168, 214, 229, 209, 221, 214, 187, 183, 187, 200, 195, 197, 207, 200, 203, 177, 155, 192, 225, 215, 197, 146, 197, 184, 154, 185, 185, 200, 218, 219, 223, 206, 185, 190, 229, 221, 178, 207, 200, 194, 209, 206, 189, 184, 172, 204, 187, 177, 184, 212, 226, 207, 188, 176, 167, 182, 198, 212, 196, 178, 169, 182, 188, 165, 152, 176, 159, 168, 221, 169, 181, 174, 164, 152, 137, 157, 181, 159, 149, 157, 167, 171, 152, 125, 108, 170, 160, 169, 196, 147, 156, 164, 137, 156, 172, 150, 131, 162, 194, 242, 225, 190, 208, 220, 197, 174, 178, 185, 188, 195, 202, 228, 193, 173, 196, 207, 211, 209, 176, 168, 190, 188, 150, 161, 195, 205, 205, 207, 206, 189, 189, 197, 212, 195, 178, 221, 207, 212, 205, 194, 165, 154, 180, 218, 197, 184, 188, 205, 209, 179, 158, 194, 187, 176, 205, 209, 182, 178, 192, 181, 206, 199, 180, 176, 173, 191, 239, 147, 162, 178, 179, 168, 158, 161, 154, 161, 149, 154, 188, 181, 142, 118, 111, 184, 173, 144, 178, 112, 172, 149, 149, 155, 189, 160, 130, 185, 201, 231, 195, 169, 184, 221, 201, 184, 184, 191, 180, 187, 204, 219, 167, 154, 184, 211, 198, 165, 161, 190, 190, 171, 154, 158, 190, 218, 200, 198, 170, 179, 203, 205, 163, 161, 188, 219, 229, 222, 229, 177, 162, 178, 209, 204, 191, 176, 188, 195, 200, 193, 165, 201, 175, 167, 176, 159, 139, 161, 206, 165, 202, 196, 174, 186, 207, 181, 171, 138, 165, 188, 196, 162, 142, 134, 140, 135, 135, 148, 205, 194, 161, 154, 164, 119, 130, 121, 162, 122, 197, 181, 164, 112, 159, 181, 167, 174, 161, 201, 193, 191, 224, 218, 199, 205, 204, 189, 170, 190, 210, 190, 166, 155, 181, 205, 192, 176, 171, 207, 171, 177, 181, 167, 191, 220, 190, 217, 176, 180, 228, 217, 159, 165, 212, 187, 211, 225, 202, 171, 174, 196, 223, 176, 184, 177, 187, 204, 194, 201, 195, 191, 187, 178, 166, 136, 143, 165, 206, 194, 205, 149, 95, 128, 169, 174, 161, 143, 161, 174, 170, 143, 137, 154, 169, 114, 129, 157, 189, 182, 186, 189, 206, 104, 130, 165, 187, 155, 189, 169, 137, 110, 156, 160, 155, 162, 148, 186, 213, 234, 248, 185, 203, 223, 226, 187, 178, 186, 219, 202, 185, 169, 180, 185, 182, 188, 178, 202, 147, 160, 215, 210, 198, 217, 188, 225, 197, 197, 224, 226, 195, 190, 208, 154, 202, 219, 171, 176, 207, 213, 183, 183, 166, 176, 177, 188, 204, 194, 178, 194, 195, 194, 167, 154, 160, 178, 179, 235, 203, 120, 78, 114, 146, 161, 195, 138, 152, 176, 157, 147, 147, 165, 201, 124, 152, 168, 167, 165, 170, 186, 206, 134, 153, 207, 182, 165, 161, 166, 108, 166, 180, 129, 108, 185, 167, 200, 218, 208, 197, 184, 208, 231, 237, 201, 173, 187, 217, 217, 192, 169, 149, 154, 166, 159, 175, 195, 116, 164, 244, 225, 206, 214, 190, 218, 195, 185, 191, 211, 205, 190, 174, 168, 207, 219, 191, 201, 238, 227, 157, 197, 175, 150, 161, 180, 182, 160, 139, 176, 190, 182, 165, 161, 183, 175, 139, 206, 171, 144, 169, 207, 167, 145, 175, 157, 173, 189, 190, 149, 132, 153, 165, 148, 186, 189, 148, 130, 151, 169, 158, 137, 152, 201, 144, 154, 138, 202, 151, 199, 173, 141, 124, 150, 208, 227, 187, 186, 221, 191, 201, 223, 223, 180, 166, 185, 176, 212, 168, 107, 121, 182, 192, 175, 177, 138, 174, 187, 196, 202, 213, 223, 214, 167, 178, 178, 171, 190, 205, 204, 181, 204, 194, 177, 187, 207, 214, 196, 183, 141, 200, 220, 189, 190, 212, 219, 189, 190, 219, 183, 180, 168, 146, 172, 156, 219, 203, 172, 153, 156, 157, 175, 179, 180, 189, 196, 137, 83, 110, 161, 184, 170, 101, 119, 175, 164, 162, 171, 121, 134, 172, 160, 177, 177, 157, 184, 147, 121, 134, 112, 194, 175, 191, 212, 212, 206, 210, 203, 222, 230, 189, 175, 190, 188, 173, 185, 187, 163, 182, 227, 224, 180, 171, 150, 181, 204, 197, 209, 210, 204, 214, 169, 174, 180, 178, 187, 204, 200, 183, 188, 171, 168, 177, 188, 204, 183, 174, 181, 208, 206, 185, 183, 202, 196, 171, 158, 192, 174, 201, 201, 177, 192, 151, 184, 162, 173, 176, 187, 185, 171, 160, 165, 165, 204, 193, 140, 115, 148, 174, 121, 86, 151, 220, 209, 186, 147, 79, 119, 182, 164, 170, 163, 144, 174, 159, 147, 149, 135, 216, 194, 181, 193, 220, 216, 163, 189, 247, 233, 186, 192, 227, 199, 168, 163, 206, 210, 195, 227, 231, 191, 163, 177, 204, 216, 204, 194, 211, 207, 194, 215, 220, 208, 197, 192, 192, 171, 146, 205, 210, 204, 200, 191, 184, 191, 177, 208, 201, 190, 183, 193, 198, 184, 157, 172, 202, 180, 202, 193, 185, 170, 115, 177, 173, 166, 187, 200, 200, 177, 150, 172, 151, 187, 227, 163, 104, 133, 178, 180, 154, 172, 203, 184, 166, 152, 116, 121, 168, 159, 164, 137, 133, 181, 181, 127, 139, 142, 202, 225, 202, 193, 187, 160, 138, 143, 242, 247, 191, 214, 234, 203, 170, 140, 211, 194, 166, 168, 195, 191, 177, 217, 215, 197, 178, 182, 197, 219, 224, 225, 230, 227, 201, 190, 180, 164, 144, 227, 234, 230, 217, 194, 188, 181, 201, 199, 177, 169, 192, 207, 180, 177, 190, 199, 211, 164, 175, 181, 164, 177, 133, 190, 190, 177, 182, 179, 189, 192, 176, 199, 170, 187, 207, 133, 102, 136, 170, 218, 183, 161, 141, 146, 172, 173, 167, 161, 185, 157, 153, 146, 123, 174, 171, 133, 147, 158, 172, 212, 218, 200, 150, 120, 128, 153, 229, 237, 177, 166, 209, 194, 161, 133, 212, 201, 124, 131, 178, 196, 202, 215, 215, 202, 191, 194, 204, 219, 211, 188, 204, 211, 202, 210, 204, 195, 184, 205, 212, 224, 208, 175, 165, 180, 190, 193, 172, 177, 195, 184, 178, 178, 218, 165, 184, 153, 171, 193, 193, 210, 170, 153, 187, 190, 161, 155, 179, 191, 207, 189, 183, 191, 170, 125, 141, 163, 160, 147, 151, 149, 150, 160, 176, 181, 198, 187, 183, 140, 157, 154, 133, 180, 162, 164, 164, 182, 182, 195, 225, 202, 155, 135, 143, 200, 202, 201, 152, 104, 157, 193, 142, 140, 215, 217, 168, 161, 180, 196, 210, 191, 200, 211, 208, 211, 228, 202, 185, 178, 198, 210, 207, 205, 204, 204, 195, 177, 197, 205, 194, 178, 178, 191, 220, 200, 191, 189, 204, 178, 160, 184, 207, 174, 193, 174, 204, 212, 200, 207, 163, 96, 148, 183, 177, 155, 166, 192, 194, 170, 165, 184, 177, 166, 184, 178, 134, 122, 160, 148, 138, 158, 161, 145, 150, 198, 189, 137, 157, 172, 142, 173, 151, 181, 170, 185, 154, 191, 198, 208, 193, 168, 144, 226, 154, 167, 157, 104, 160, 215, 128, 147, 219, 234, 191, 184, 180, 184, 205, 190, 202, 210, 205, 210, 214, 204, 185, 179, 211, 218, 218, 200, 197, 188, 173, 191, 205, 217, 207, 185, 190, 211, 234, 218, 211, 195, 188, 185, 174, 181, 192, 189, 226, 192, 205, 185, 176, 183, 147, 92, 145, 189, 174, 166, 176, 174, 175, 168, 143, 159, 177, 187, 168, 163, 100, 137, 184, 165, 131, 158, 131, 124, 160, 167, 170, 140, 172, 166, 146, 170, 156, 178, 141, 151, 114, 198, 170, 191, 234, 202, 127, 225, 105, 153, 195, 133, 210, 237, 111, 163, 219, 222, 198, 171, 167, 160, 191, 231, 221, 184, 183, 185, 200, 209, 191, 173, 190, 209, 208, 200, 205, 202, 194, 221, 215, 212, 209, 194, 188, 202, 222, 210, 212, 185, 165, 178, 200, 202, 192, 184, 205, 168, 163, 168, 151, 201, 173, 141, 177, 192, 177, 169, 173, 186, 163, 216, 124, 116, 172, 175, 147, 105, 80, 88, 160, 167, 165, 190, 191, 160, 167, 143, 160, 137, 177, 168, 137, 170, 167, 165, 131, 142, 102, 186, 167, 190, 247, 205, 132, 98, 111, 190, 193, 197, 242, 207, 150, 160, 229, 207, 158, 197, 191, 185, 242, 226, 190, 221, 217, 174, 194, 228, 204, 186, 207, 220, 198, 177, 161, 171, 198, 220, 215, 207, 207, 207, 190, 195, 212, 195, 185, 192, 198, 205, 180, 135, 98, 195, 178, 168, 180, 176, 164, 178, 211, 168, 188, 190, 161, 155, 181, 181, 169, 140, 143, 156, 160, 155, 133, 158, 190, 98, 127, 158, 164, 165, 179, 170, 158, 115, 150, 167, 194, 178, 143, 134, 138, 194, 174, 141, 187, 147, 201, 237, 206, 204, 220, 107, 103, 190, 205, 183, 203, 191, 195, 168, 219, 180, 138, 167, 170, 165, 212, 228, 212, 220, 181, 145, 204, 227, 180, 195, 186, 207, 205, 190, 168, 176, 207, 205, 197, 207, 209, 219, 211, 200, 205, 208, 190, 180, 181, 183, 183, 154, 127, 163, 171, 178, 158, 154, 154, 167, 177, 177, 207, 220, 216, 213, 214, 203, 183, 150, 157, 159, 165, 153, 148, 170, 177, 135, 156, 158, 150, 167, 167, 150, 134, 160, 170, 175, 161, 141, 133, 141, 161, 175, 158, 155, 187, 137, 201, 232, 178, 175, 237, 126, 99, 169, 197, 194, 212, 191, 211, 204, 202, 202, 178, 178, 188, 191, 192, 208, 217, 202, 147, 125, 201, 234, 180, 207, 184, 178, 215, 214, 191, 183, 202, 204, 195, 190, 188, 197, 217, 212, 202, 224, 197, 161, 164, 190, 200, 185, 160, 173, 184, 190, 181, 165, 172, 187, 195, 204, 200, 173, 168, 158, 178, 203, 203, 177, 201, 193, 170, 177, 199, 192, 183, 145, 164, 168, 165, 154, 143, 147, 131, 160, 177, 178, 155, 142, 142, 166, 195, 144, 168, 151, 192, 164, 148, 162, 177, 192, 198, 164, 113, 161, 186, 204, 239, 200, 174, 187, 178, 191, 202, 197, 200, 191, 157, 174, 202, 204, 164, 155, 200, 217, 204, 220, 172, 173, 222, 224, 197, 180, 190, 207, 202, 175, 161, 173, 208, 225, 211, 214, 173, 154, 150, 181, 212, 212, 200, 171, 177, 183, 199, 190, 199, 210, 218, 194, 172, 147, 123, 129, 158, 175, 189, 194, 211, 212, 172, 177, 209, 205, 172, 157, 169, 172, 156, 165, 165, 157, 148, 128, 158, 186, 174, 156, 163, 158, 166, 155, 173, 181, 182, 183, 137, 145, 201, 235, 208, 153, 164, 218, 179, 173, 245, 218, 157, 167, 141, 154, 183, 175, 178, 174, 144, 170, 188, 205, 217, 194, 185, 194, 232, 219, 175, 176, 214, 227, 197, 183, 193, 211, 204, 187, 170, 190, 205, 217, 210, 202, 178, 151, 165, 194, 216, 207, 191, 208, 150, 138, 187, 198, 181, 181, 216, 172, 154, 158, 165, 183, 181, 156, 145, 170, 192, 179, 154, 150, 183, 181, 148, 139, 164, 167, 154, 157, 161, 154, 135, 131, 157, 174, 170, 149, 132, 129, 125, 152, 184, 201, 208, 192, 192, 186, 181, 201, 237, 134, 190, 244, 188, 142, 207, 217, 184, 198, 189, 167, 168, 187, 185, 177, 183, 181, 197, 208, 221, 221, 172, 161, 221, 192, 190, 188, 201, 208, 200, 188, 197, 209, 205, 207, 219, 210, 200, 190, 194, 201, 185, 168, 170, 187, 191, 187, 177, 230, 170, 144, 157, 174, 161, 158, 196, 196, 172, 159, 176, 201, 179, 166, 147, 163, 177, 182, 157, 155, 173, 177, 160, 151, 162, 159, 158, 146, 144, 133, 116, 137, 157, 160, 136, 146, 172, 172, 155, 134, 174, 180, 178, 204, 227, 197, 151, 160, 210, 174, 174, 234, 167, 128, 204, 208, 197, 225, 228, 207, 201, 224, 185, 150, 180, 197, 204, 187, 178, 202, 182, 161, 211, 158, 183, 204, 191, 184, 195, 211, 204, 225, 204, 197, 215, 193, 154, 155, 194, 181, 181, 170, 168, 175, 184, 185, 184, 211, 212, 184, 138, 131, 153, 180, 183, 240, 194, 146, 146, 167, 170, 169, 163, 173, 178, 184, 181, 177, 178, 184, 193, 153, 166, 174, 156, 154, 154, 129, 106, 163, 163, 153, 137, 154, 193, 201, 171, 121, 156, 172, 181, 192, 198, 184, 157, 173, 181, 218, 164, 158, 129, 153, 207, 210, 195, 169, 227, 202, 193, 232, 162, 76, 121, 185, 227, 153, 114, 172, 204, 187, 204, 134, 186, 209, 181, 167, 191, 231, 232, 246, 191, 174, 185, 153, 103, 116, 208, 165, 158, 161, 170, 163, 175, 189, 196, 178, 229, 211, 107, 75, 141, 200, 191, 221, 177, 159, 182, 195, 187, 157, 142, 163, 166, 174, 192, 177, 163, 191, 210, 158, 166, 186, 172, 171, 164, 145, 125, 184, 187, 158, 135, 147, 171, 141, 90, 144, 190, 189, 192, 184, 219, 174, 131, 228, 218, 178, 117, 153, 184, 178, 204, 224, 188, 164, 190, 242, 128, 140, 186, 98, 178, 198, 211, 214, 214, 200, 197, 192, 189, 175, 186, 185, 193, 211, 210, 194, 188, 208, 185, 170, 178, 188, 188, 188, 177, 177, 173, 196, 189, 143, 138, 170, 191, 149, 212, 208, 152, 143, 171, 184, 192, 204, 167, 142, 172, 165, 140, 137, 154, 157, 165, 181, 163, 162, 168, 173, 169, 170, 152, 156, 202, 183, 128, 138, 195, 150, 155, 138, 132, 180, 174, 123, 90, 180, 193, 244, 205, 195, 244, 231, 181, 212, 171, 117, 134, 197, 219, 153, 163, 210, 192, 233, 204, 198, 134, 177, 215, 153, 199, 205, 211, 212, 185, 177, 183, 178, 157, 210, 200, 188, 184, 197, 200, 185, 173, 190, 175, 168, 178, 197, 208, 182, 164, 158, 147, 163, 168, 143, 153, 174, 161, 128, 195, 200, 181, 184, 167, 150, 160, 184, 131, 130, 164, 189, 167, 177, 187, 154, 163, 160, 158, 170, 172, 175, 176, 208, 178, 176, 189, 170, 124, 119, 163, 156, 167, 173, 164, 160, 163, 136, 97, 141, 143, 188, 161, 190, 246, 230, 185, 195, 148, 138, 140, 194, 201, 175, 198, 220, 158, 187, 191, 197, 202, 237, 220, 154, 173, 197, 214, 197, 158, 165, 195, 177, 134, 210, 178, 173, 180, 191, 190, 183, 159, 192, 185, 161, 163, 176, 205, 195, 147, 145, 138, 155, 155, 153, 189, 201, 150, 128, 168, 190, 188, 196, 174, 161, 173, 164, 148, 147, 152, 178, 189, 177, 169, 183, 159, 150, 184, 178, 150, 166, 186, 184, 170, 162, 161, 151, 147, 168, 173, 186, 167, 190, 182, 119, 129, 161, 129, 141, 121, 181, 168, 176, 217, 194, 174, 190, 155, 167, 153, 190, 207, 178, 211, 211, 120, 135, 174, 192, 247, 232, 147, 151, 168, 200, 208, 181, 159, 177, 202, 202, 161, 155, 168, 178, 185, 190, 185, 194, 190, 212, 207, 159, 133, 147, 191, 177, 138, 104, 124, 147, 151, 163, 204, 210, 151, 163, 178, 167, 177, 191, 170, 177, 218, 133, 158, 163, 154, 160, 182, 164, 154, 222, 172, 167, 180, 177, 154, 155, 176, 176, 172, 161, 147, 167, 195, 193, 206, 197, 164, 194, 167, 107, 134, 203, 162, 127, 140, 195, 168, 200, 200, 174, 180, 187, 198, 170, 178, 228, 236, 160, 174, 198, 162, 163, 165, 170, 217, 167, 123, 170, 204, 217, 209, 181, 160, 172, 202, 195, 183, 147, 167, 198, 197, 191, 176, 173, 185, 187, 201, 178, 144, 161, 195, 171, 109, 87, 102, 145, 148, 148, 174, 182, 150, 188, 184, 160, 154, 175, 168, 172, 212, 103, 148, 174, 170, 161, 158, 182, 186, 227, 178, 165, 189, 191, 158, 155, 184, 204, 204, 178, 164, 159, 178, 191, 164, 191, 133, 167, 187, 116, 154, 205, 152, 105, 147, 206, 128, 208, 197, 159, 178, 168, 203, 191, 163, 204, 217, 188, 191, 200, 148, 182, 133, 163, 201, 192, 180, 188, 208, 225, 204, 171, 171, 163, 175, 177, 183, 175, 201, 228, 218, 184, 153, 154, 160, 150, 174, 185, 192, 205, 201, 167, 113, 105, 113, 150, 184, 155, 151, 178, 163, 185, 195, 177, 153, 170, 160, 131, 143, 141, 167, 189, 191, 175, 155, 173, 190, 195, 167, 148, 178, 189, 176, 175, 186, 188, 199, 185, 178, 169, 165, 175, 163, 157, 134, 181, 189, 159, 177, 185, 125, 103, 152, 188, 108, 205, 185, 155, 161, 161, 200, 211, 120, 132, 216, 232, 222, 211, 120, 140, 77, 184, 202, 191, 231, 187, 192, 238, 192, 164, 180, 174, 145, 163, 214, 202, 210, 207, 205, 191, 173, 168, 168, 164, 171, 200, 224, 221, 207, 168, 144, 150, 138, 174, 208, 178, 165, 174, 185, 170, 204, 177, 143, 160, 163, 133, 114, 199, 177, 184, 198, 189, 154, 158, 170, 167, 160, 154, 158, 176, 186, 177, 164, 145, 148, 161, 175, 165, 161, 158, 182, 164, 182, 188, 171, 169, 164, 154, 135, 135, 147, 177, 115, 211, 181, 154, 180, 170, 215, 153, 100, 148, 236, 229, 198, 184, 155, 105, 48, 200, 171, 115, 181, 188, 247, 247, 170, 163, 183, 187, 151, 174, 242, 200, 188, 174, 183, 197, 200, 214, 198, 208, 204, 209, 224, 207, 190, 171, 175, 184, 129, 164, 217, 194, 158, 155, 164, 189, 202, 168, 135, 153, 179, 158, 131, 187, 148, 145, 177, 205, 175, 170, 190, 159, 157, 150, 147, 169, 181, 166, 149, 114, 124, 148, 174, 162, 135, 144, 158, 228, 229, 186, 144, 132, 127, 154, 168, 161, 140, 142, 120, 201, 176, 178, 209, 212, 197, 132, 118, 140, 243, 229, 164, 210, 154, 135, 168, 211, 176, 107, 187, 234, 133, 212, 236, 217, 202, 170, 131, 145, 227, 193, 184, 188, 164, 151, 144, 159, 197, 185, 212, 229, 200, 195, 202, 188, 163, 188, 167, 212, 175, 182, 218, 177, 214, 147, 155, 181, 175, 170, 187, 168, 128, 130, 151, 155, 210, 150, 153, 140, 187, 142, 146, 158, 182, 164, 130, 157, 191, 141, 174, 139, 169, 158, 162, 108, 188, 185, 181, 186, 152, 137, 132, 137, 149, 157, 134, 132, 147, 225, 201, 165, 184, 195, 185, 110, 117, 165, 235, 207, 154, 180, 140, 110, 103, 173, 190, 117, 148, 202, 167, 185, 197, 202, 208, 191, 145, 136, 204, 195, 184, 168, 154, 151, 151, 181, 194, 204, 204, 197, 173, 144, 117, 139, 163, 165, 131, 177, 177, 192, 218, 191, 211, 147, 194, 196, 158, 164, 187, 143, 86, 161, 183, 160, 182, 133, 155, 143, 185, 176, 159, 157, 158, 155, 156, 176, 190, 128, 181, 129, 184, 169, 169, 118, 189, 199, 185, 174, 181, 155, 125, 132, 161, 159, 159, 144, 162, 187, 183, 221, 238, 210, 177, 135, 125, 212, 228, 188, 181, 144, 127, 208, 142, 188, 224, 141, 107, 178, 221, 197, 172, 165, 192, 204, 153, 138, 191, 208, 207, 170, 153, 177, 212, 217, 195, 192, 185, 180, 184, 140, 104, 148, 214, 199, 178, 197, 181, 191, 217, 182, 194, 147, 230, 208, 150, 170, 191, 141, 87, 187, 201, 158, 181, 124, 177, 167, 179, 192, 179, 158, 148, 169, 187, 174, 168, 122, 167, 158, 186, 164, 168, 134, 178, 185, 137, 137, 184, 155, 89, 101, 154, 135, 169, 162, 158, 163, 187, 211, 230, 222, 191, 148, 124, 216, 231, 198, 160, 96, 148, 234, 173, 164, 188, 113, 61, 134, 205, 217, 210, 187, 185, 185, 141, 145, 207, 202, 218, 202, 168, 180, 225, 234, 176, 190, 174, 191, 187, 161, 148, 171, 211, 192, 201, 207, 196, 191, 187, 163, 164, 135, 235, 201, 158, 204, 198, 133, 141, 194, 218, 191, 175, 135, 175, 177, 191, 174, 173, 168, 160, 166, 189, 184, 144, 121, 174, 144, 165, 134, 171, 134, 162, 174, 137, 119, 134, 118, 77, 84, 144, 128, 159, 161, 138, 221, 188, 134, 115, 167, 239, 148, 103, 198, 220, 199, 144, 56, 148, 200, 179, 181, 160, 111, 104, 143, 188, 211, 228, 211, 201, 198, 168, 171, 232, 181, 201, 198, 176, 178, 208, 200, 165, 176, 191, 170, 137, 144, 163, 155, 150, 151, 183, 194, 199, 175, 153, 167, 170, 155, 204, 177, 172, 218, 174, 121, 175, 162, 200, 184, 195, 135, 172, 164, 190, 125, 164, 184, 175, 189, 199, 164, 120, 142, 165, 154, 178, 115, 152, 147, 138, 178, 162, 145, 129, 107, 108, 132, 159, 141, 182, 152, 130, 156, 148, 120, 114, 161, 237, 155, 138, 180, 210, 220, 144, 96, 173, 181, 220, 217, 178, 161, 180, 157, 121, 161, 230, 238, 219, 211, 207, 210, 220, 169, 201, 215, 192, 180, 192, 190, 171, 171, 173, 138, 104, 124, 184, 190, 148, 177, 201, 181, 190, 167, 144, 180, 181, 175, 177, 147, 175, 196, 138, 107, 175, 147, 170, 153, 196, 143, 158, 126, 155, 122, 139, 160, 188, 189, 187, 164, 120, 149, 172, 171, 172, 107, 171, 162, 140, 178, 174, 154, 128, 112, 128, 158, 172, 178, 196, 159, 128, 95, 139, 183, 181, 192, 194, 128, 204, 189, 197, 237, 187, 174, 178, 157, 205, 184, 135, 158, 175, 128, 64, 145, 224, 243, 185, 202, 234, 210, 192, 190, 193, 198, 198, 194, 190, 201, 205, 180, 140, 121, 140, 184, 212, 215, 202, 194, 213, 145, 148, 147, 134, 194, 188, 170, 165, 155, 167, 150, 124, 131, 164, 172, 144, 125, 198, 165, 151, 105, 128, 144, 124, 135, 156, 189, 196, 162, 160, 117, 161, 172, 190, 111, 182, 174, 132, 168, 142, 147, 138, 125, 125, 144, 179, 188, 187, 148, 115, 161, 192, 207, 188, 167, 184, 108, 230, 174, 180, 226, 205, 219, 170, 174, 200, 151, 117, 171, 220, 182, 126, 164, 247, 217, 143, 161, 228, 221, 177, 191, 177, 174, 183, 184, 180, 194, 208, 185, 160, 149, 195, 230, 215, 192, 187, 175, 160, 64, 81, 115, 131, 214, 191, 181, 154, 171, 158, 131, 147, 167, 155, 211, 148, 102, 189, 187, 165, 95, 114, 187, 120, 93, 134, 190, 193, 186, 183, 80, 132, 165, 194, 117, 181, 181, 115, 164, 132, 137, 175, 164, 118, 132, 188, 151, 158, 123, 91, 207, 232, 146, 154, 232, 167, 201, 164, 155, 155, 164, 201, 202, 151, 117, 163, 204, 187, 182, 235, 241, 165, 128, 180, 214, 188, 187, 215, 215, 189, 201, 184, 175, 174, 180, 178, 180, 190, 202, 243, 210, 178, 162, 184, 225, 208, 194, 109, 107, 158, 171, 197, 205, 165, 149, 101, 144, 151, 149, 172, 177, 204, 184, 158, 150, 197, 204, 133, 114, 173, 176, 144, 71, 151, 193, 153, 186, 145, 130, 130, 181, 161, 171, 179, 127, 127, 101, 195, 166, 118, 149, 149, 119, 128, 171, 148, 129, 104, 178, 222, 135, 130, 207, 177, 120, 142, 180, 194, 169, 110, 135, 196, 128, 157, 188, 178, 153, 190, 184, 120, 178, 197, 195, 204, 204, 204, 198, 208, 221, 206, 192, 174, 173, 185, 197, 195, 185, 219, 185, 165, 157, 173, 220, 181, 174, 136, 133, 172, 187, 205, 205, 185, 191, 117, 134, 132, 147, 187, 188, 212, 187, 178, 178, 205, 213, 161, 119, 157, 198, 142, 49, 138, 205, 200, 194, 156, 146, 118, 174, 190, 171, 155, 91, 128, 123, 181, 165, 137, 145, 158, 140, 164, 187, 169, 155, 142, 165, 204, 163, 132, 205, 180, 144, 160, 188, 214, 183, 66, 60, 220, 160, 157, 178, 178, 145, 164, 185, 163, 211, 188, 185, 201, 197, 192, 190, 204, 210, 192, 171, 162, 165, 181, 187, 185, 173, 205, 158, 168, 175, 167, 211, 173, 123, 160, 174, 180, 195, 205, 181, 190, 163, 83, 93, 107, 125, 195, 191, 200, 158, 195, 179, 184, 200, 184, 164, 173, 193, 145, 98, 153, 217, 197, 196, 172, 156, 131, 171, 174, 169, 159, 123, 151, 151, 179, 161, 135, 159, 142, 148, 179, 167, 135, 150, 147, 168, 207, 173, 174, 224, 183, 209, 212, 163, 178, 205, 110, 80, 210, 217, 184, 188, 181, 148, 157, 198, 227, 215, 194, 183, 194, 192, 185, 174, 158, 189, 181, 174, 179, 184, 192, 187, 181, 164, 183, 148, 164, 178, 175, 204, 160, 94, 170, 187, 171, 212, 198, 156, 153, 115, 58, 105, 118, 128, 187, 182, 182, 198, 191, 160, 154, 170, 154, 164, 190, 153, 150, 167, 215, 188, 139, 144, 167, 191, 172, 189, 162, 141, 178, 161, 189, 165, 158, 145, 154, 137, 125, 132, 159, 156, 131, 142, 139, 177, 170, 181, 228, 243, 171, 192, 204, 143, 146, 219, 191, 135, 195, 210, 190, 197, 198, 151, 143, 177, 193, 174, 210, 204, 174, 184, 214, 185, 105, 161, 171, 191, 208, 211, 201, 178, 173, 159, 167, 141, 170, 181, 167, 178, 163, 135, 187, 165, 143, 188, 191, 149, 167, 131, 105, 159, 148, 140, 185, 184, 206, 216, 189, 150, 158, 170, 138, 131, 190, 136, 140, 177, 193, 166, 106, 108, 154, 184, 194, 211, 141, 115, 169, 159, 154, 125, 121, 158, 152, 148, 132, 139, 149, 181, 173, 162, 157, 149, 155, 188, 211, 241, 154, 100, 146, 144, 154, 214, 212, 178, 181, 118, 130, 173, 201, 153, 147, 164, 169, 145, 208, 215, 180, 188, 230, 183, 103, 128, 161, 198, 198, 191, 177, 157, 153, 190, 191, 157, 204, 202, 167, 187, 177, 204, 198, 155, 125, 157, 184, 185, 190, 188, 171, 210, 157, 110, 148, 178, 212, 177, 167, 148, 194, 211, 157, 144, 200, 174, 138, 167, 170, 172, 160, 130, 194, 102, 142, 191, 132, 113, 169, 142, 135, 103, 112, 144, 171, 145, 142, 159, 141, 161, 150, 158, 147, 154, 173, 200, 178, 185, 173, 95, 121, 159, 180, 192, 207, 181, 161, 97, 126, 183, 191, 155, 141, 168, 160, 134, 184, 207, 191, 191, 208, 181, 105, 153, 174, 194, 186, 154, 144, 143, 159, 207, 198, 175, 208, 202, 157, 179, 184, 222, 170, 154, 168, 165, 180, 214, 212, 188, 184, 219, 159, 95, 127, 155, 188, 148, 160, 162, 191, 211, 191, 171, 205, 157, 109, 169, 181, 180, 166, 140, 194, 105, 109, 149, 132, 118, 141, 125, 169, 109, 114, 165, 175, 125, 149, 169, 114, 119, 121, 138, 148, 167, 224, 222, 130, 141, 197, 181, 155, 164, 190, 201, 195, 172, 148, 158, 190, 235, 194, 135, 137, 157, 144, 130, 150, 172, 175, 196, 189, 155, 127, 202, 221, 222, 191, 145, 124, 168, 195, 194, 181, 160, 194, 187, 114, 145, 173, 185, 143, 172, 208, 178, 181, 202, 202, 171, 171, 224, 171, 117, 143, 154, 168, 170, 213, 189, 155, 173, 184, 165, 160, 59, 80, 191, 201, 155, 120, 80, 164, 217, 144, 158, 149, 115, 89, 89, 190, 122, 129, 175, 155, 101, 134, 152, 88, 135, 147, 167, 153, 195, 217, 108, 142, 217, 158, 151, 173, 174, 144, 159, 182, 201, 170, 190, 190, 155, 221, 193, 197, 131, 173, 157, 202, 198, 164, 186, 198, 179, 172, 181, 196, 231, 224, 175, 158, 178, 170, 208, 218, 168, 124, 133, 148, 164, 194, 184, 140, 160, 183, 161, 195, 200, 117, 161, 205, 211, 157, 117, 142, 148, 141, 170, 194, 204, 185, 187, 185, 148, 157, 102, 157, 189, 167, 159, 133, 110, 125, 188, 205, 172, 146, 175, 154, 115, 147, 202, 165, 138, 122, 114, 139, 148, 115, 144, 151, 129, 88, 185, 141, 79, 142, 200, 147, 173, 188, 180, 168, 171, 185, 195, 187, 170, 170, 181, 202, 174, 171, 178, 198, 121, 173, 187, 165, 197, 217, 207, 191, 144, 185, 223, 217, 197, 181, 173, 200, 149, 172, 200, 196, 170, 177, 183, 187, 177, 150, 173, 181, 144, 165, 178, 133, 125, 157, 154, 122, 115, 147, 175, 181, 191, 179, 172, 165, 143, 167, 181, 165, 82, 150, 179, 178, 173, 137, 131, 158, 183, 120, 115, 188, 195, 80, 23, 86, 187, 158, 168, 134, 130, 161, 161, 124, 98, 132, 138, 129, 180, 121, 120, 178, 202, 158, 194, 171, 168, 183, 195, 201, 197, 195, 150, 140, 191, 189, 162, 147, 205, 198, 152, 201, 195, 181, 202, 198, 180, 170, 140, 190, 201, 220, 234, 180, 140, 200, 181, 173, 177, 158, 104, 138, 200, 184, 168, 164, 192, 182, 146, 158, 188, 159, 125, 112, 105, 115, 134, 156, 183, 201, 178, 150, 165, 154, 135, 170, 201, 170, 97, 163, 192, 189, 177, 136, 131, 166, 165, 112, 112, 156, 137, 107, 118, 117, 159, 149, 162, 157, 141, 152, 169, 142, 149, 131, 102, 102, 173, 168, 202, 224, 200, 212, 194, 158, 153, 188, 209, 184, 182, 209, 175, 121, 214, 189, 153, 120, 210, 164, 190, 210, 204, 174, 192, 185, 153, 153, 145, 204, 204, 194, 237, 154, 108, 171, 234, 161, 175, 131, 65, 114, 204, 178, 175, 181, 194, 174, 170, 184, 190, 190, 144, 127, 125, 151, 167, 168, 165, 172, 160, 128, 175, 187, 121, 155, 204, 146, 123, 184, 208, 195, 169, 114, 105, 146, 165, 131, 148, 164, 132, 149, 195, 200, 142, 142, 162, 159, 132, 140, 165, 141, 178, 134, 81, 65, 171, 191, 246, 230, 188, 235, 214, 168, 161, 198, 205, 178, 170, 195, 180, 134, 207, 201, 170, 153, 225, 157, 164, 188, 180, 164, 187, 187, 171, 185, 147, 191, 167, 167, 205, 155, 93, 166, 173, 142, 168, 194, 138, 170, 211, 151, 184, 191, 170, 171, 181, 188, 191, 174, 171, 153, 155, 185, 192, 178, 138, 118, 155, 133, 190, 196, 121, 133, 194, 158, 154, 193, 198, 191, 155, 93, 91, 137, 183, 101, 151, 241, 187, 90, 128, 174, 144, 151, 161, 149, 123, 135, 155, 145, 124, 107, 95, 95, 163, 185, 228, 171, 151, 242, 220, 188, 187, 211, 197, 168, 151, 178, 174, 163, 210, 203, 180, 186, 220, 175, 162, 191, 173, 179, 200, 181, 177, 187, 127, 174, 151, 157, 184, 174, 143, 178, 142, 138, 190, 209, 161, 175, 178, 124, 190, 201, 158, 146, 184, 194, 146, 138, 163, 163, 164, 185, 200, 188, 149, 111, 160, 150, 183, 160, 111, 128, 183, 171, 162, 176, 175, 167, 149, 102, 93, 157, 158, 141, 171, 172, 123, 94, 115, 151, 156, 152, 172, 149, 124, 142, 165, 145, 102, 108, 116, 116, 176, 190, 191, 151, 151, 207, 192, 195, 197, 205, 184, 145, 161, 174, 165, 195, 201, 196, 155, 171, 184, 186, 194, 208, 201, 201, 207, 183, 148, 158, 161, 181, 174, 164, 178, 185, 175, 144, 132, 163, 192, 176, 163, 144, 154, 154, 168, 191, 151, 137, 192, 190, 144, 135, 159, 171, 167, 163, 185, 195, 188, 149, 155, 160, 155, 138, 133, 145, 162, 157, 154, 160, 145, 150, 150, 123, 122, 170, 134, 200, 169, 77, 79, 144, 168, 141, 148, 145, 152, 165, 135, 149, 165, 132, 132, 125, 119, 116, 211, 204, 195, 167, 167, 188, 155, 170, 190, 183, 168, 156, 158, 166, 161, 226, 197, 187, 114, 148, 131, 189, 175, 210, 218, 221, 230, 207, 148, 148, 196, 215, 225, 188, 157, 185, 173, 98, 80, 154, 212, 202, 175, 168, 165, 209, 138, 181, 151, 140, 195, 202, 159, 163, 176, 185, 173, 140, 160, 219, 222, 190, 127, 135, 138, 127, 164, 177, 141, 130, 155, 155, 133, 145, 149, 122, 133, 201, 144, 174, 144, 127, 154, 144, 107, 121, 124, 107, 153, 150, 133, 151, 145, 109, 129, 138, 141, 149, 171, 197, 226, 209, 176, 190, 173, 195, 205, 207, 185, 177, 171, 187, 211, 165, 140, 177, 190, 173, 163, 161, 143, 207, 194, 158, 160, 175, 179, 192, 201, 213, 230, 246, 220, 195, 147, 114, 103, 177, 210, 180, 148, 171, 207, 211, 178, 180, 205, 187, 154, 168, 160, 142, 197, 167, 145, 168, 181, 180, 151, 147, 96, 123, 144, 161, 173, 177, 160, 144, 166, 165, 144, 160, 157, 93, 79, 121, 165, 178, 155, 134, 115, 78, 89, 157, 109, 81, 155, 141, 85, 94, 80, 145, 165, 151, 99, 130, 167, 192, 195, 177, 168, 217, 205, 225, 240, 238, 200, 195, 195, 218, 204, 222, 230, 222, 170, 117, 149, 202, 171, 220, 204, 161, 167, 185, 190, 191, 165, 188, 205, 215, 210, 187, 165, 144, 151, 201, 217, 184, 164, 174, 191, 188, 175, 195, 205, 165, 94, 98, 163, 165, 154, 167, 184, 185, 153, 121, 156, 185, 163, 175, 171, 156, 156, 181, 188, 171, 176, 157, 150, 163, 166, 123, 103, 178, 151, 128, 132, 135, 125, 98, 125, 155, 92, 62, 156, 146, 98, 78, 60, 99, 148, 164, 129, 159, 182, 195, 191, 159, 178, 234, 174, 200, 221, 188, 147, 130, 161, 188, 187, 214, 246, 224, 144, 91, 115, 183, 184, 210, 184, 155, 173, 190, 181, 184, 165, 174, 181, 189, 191, 180, 178, 167, 200, 202, 201, 185, 180, 181, 174, 170, 205, 148, 127, 121, 125, 161, 163, 124, 144, 151, 155, 165, 155, 140, 161, 191, 171, 171, 173, 151, 144, 155, 170, 155, 151, 159, 140, 151, 157, 135, 133, 187, 165, 114, 142, 169, 125, 114, 137, 127, 91, 67, 147, 176, 144, 132, 92, 109, 108, 147, 140, 152, 218, 211, 215, 185, 197, 243, 198, 221, 228, 197, 170, 151, 173, 205, 185, 188, 180, 188, 161, 134, 122, 149, 180, 197, 188, 171, 192, 192, 172, 183, 184, 172, 179, 177, 171, 170, 170, 173, 210, 175, 157, 189, 207, 189, 181, 183, 204, 147, 128, 147, 176, 194, 178, 103, 134, 137, 137, 137, 143, 163, 167, 183, 122, 135, 143, 151, 167, 156, 139, 134, 144, 137, 132, 153, 162, 126, 123, 190, 169, 108, 152, 195, 124, 118, 150, 135, 129, 90, 135, 174, 176, 159, 128, 105, 108, 150, 122, 119, 221, 214, 218, 205, 185, 212, 246, 246, 232, 221, 204, 200, 205, 205, 228, 173, 159, 191, 210, 188, 154, 149, 195, 211, 194, 191, 220, 192, 159, 171, 194, 191, 180, 165, 169, 170, 179, 180, 217, 171, 152, 186, 207, 201, 188, 194, 198, 185, 214, 212, 188, 161, 158, 145, 71, 125, 161, 151, 113, 101, 156, 204, 108, 117, 154, 171, 167, 156, 139, 121, 146, 159, 159, 170, 175, 138, 129, 161, 129, 94, 155, 187, 130, 120, 177, 158, 180, 107, 79, 127, 142, 134, 113, 64, 129, 165, 121, 118, 206, 194, 208, 200, 173, 176, 220, 224, 220, 208, 213, 200, 200, 195, 198, 187, 190, 215, 214, 180, 165, 183, 200, 201, 197, 192, 198, 149, 117, 137, 197, 188, 183, 173, 177, 181, 188, 194, 218, 181, 163, 191, 203, 203, 198, 192, 190, 145, 155, 171, 167, 163, 184, 171, 94, 117, 133, 123, 113, 120, 133, 140, 138, 143, 144, 160, 163, 133, 125, 127, 126, 153, 156, 163, 167, 137, 117, 143, 135, 104, 156, 177, 108, 103, 145, 165, 160, 80, 39, 90, 128, 125, 139, 84, 105, 133, 98, 132, 213, 193, 198, 193, 177, 189, 201, 220, 230, 230, 225, 213, 208, 198, 183, 173, 202, 220, 209, 180, 174, 195, 205, 198, 187, 181, 177, 124, 105, 150, 190, 190, 187, 180, 182, 185, 189, 193, 191, 175, 165, 181, 185, 189, 198, 202, 180, 104, 86, 123, 161, 205, 200, 173, 163, 141, 114, 115, 150, 165, 120, 70, 160, 144, 151, 160, 154, 125, 120, 125, 114, 133, 141, 144, 153, 133, 121, 157, 162, 152, 153, 158, 128, 99, 124, 158, 139, 98, 51, 138, 145, 142, 172, 118, 81, 102, 89, 127, 223, 189, 190, 199, 208, 221, 165, 173, 213, 221, 199, 184, 183, 185, 165, 171, 200, 224, 226, 205, 202, 217, 217, 201, 190, 195, 188, 140, 137, 197, 202, 204, 191, 187, 184, 184, 178, 182, 151, 158, 164, 154, 157, 180, 194, 197, 201, 160, 150, 171, 168, 171, 187, 167, 178, 180, 177, 154, 142, 149, 124, 81, 158, 153, 161, 180, 178, 149, 125, 127, 117, 149, 150, 144, 163, 155, 149, 183, 175, 167, 136, 139, 159, 128, 146, 191, 160, 147, 107, 177, 174, 114, 166, 114, 108, 116, 78, 115
};

GlGrowthPattern* gl_new_fern_moss_pattern()
{
	GlGrowthPattern*	pattern = new GlGrowthPattern(gFernMossData, gFernMossW, gFernMossH);
	if (!pattern) return 0;
	if (pattern->InitCheck() == B_OK) return pattern;
	delete pattern;
	return 0;
}


const int32	gHairCapMossW = 98;
const int32 gHairCapMossH = 99;
const uint8 gHairCapMossData[] = {
	88, 95, 102, 72, 68, 71, 66, 48, 49, 81, 127, 100, 101, 78, 78, 112, 106, 70, 72, 73, 56, 36, 32, 32, 30, 45, 68, 83, 128, 142, 105, 68, 62, 70, 81, 40, 40, 52, 67, 77, 82, 79, 75, 83, 76, 76, 75, 69, 56, 66, 86, 112, 123, 124, 104, 87, 88, 95, 93, 116, 151, 171, 139, 77, 37, 36, 57, 83, 114, 139, 134, 104, 68, 50, 42, 61, 112, 144, 128, 114, 127, 162, 178, 209, 217, 204, 163, 115, 83, 56, 44, 70, 75, 101, 127, 100, 57, 63, 103, 118, 118, 109, 80, 96, 110, 86, 71, 55, 56, 44, 52, 76, 108, 105, 101, 75, 80, 125, 122, 84, 83, 78, 57, 37, 33, 32, 32, 49, 71, 77, 104, 111, 79, 51, 55, 70, 85, 75, 72, 70, 75, 78, 78, 68, 58, 52, 52, 61, 78, 77, 64, 70, 79, 174, 167, 137, 92, 85, 99, 112, 119, 112, 166, 199, 167, 87, 37, 42, 65, 73, 100, 138, 144, 114, 82, 61, 56, 56, 100, 122, 118, 107, 133, 177, 200, 176, 191, 185, 139, 97, 68, 52, 39, 41, 56, 106, 150, 134, 79, 57, 72, 100, 107, 101, 60, 84, 103, 103, 78, 48, 44, 41, 49, 68, 95, 114, 115, 89, 95, 142, 143, 105, 99, 85, 59, 40, 35, 34, 37, 56, 79, 81, 90, 85, 63, 49, 62, 81, 89, 98, 90, 76, 68, 62, 58, 48, 40, 32, 42, 61, 90, 95, 86, 88, 96, 165, 154, 110, 72, 69, 90, 108, 113, 109, 143, 161, 147, 99, 62, 49, 53, 58, 85, 127, 142, 123, 96, 68, 58, 52, 77, 84, 83, 77, 109, 153, 177, 154, 170, 165, 114, 66, 45, 40, 32, 28, 41, 93, 143, 142, 97, 59, 56, 91, 100, 99, 45, 70, 89, 107, 84, 63, 48, 42, 41, 62, 91, 102, 116, 98, 100, 127, 119, 93, 98, 88, 62, 42, 41, 40, 44, 64, 92, 95, 95, 85, 77, 72, 79, 93, 92, 87, 73, 54, 43, 34, 30, 25, 25, 26, 35, 52, 78, 84, 77, 78, 95, 93, 95, 82, 63, 56, 79, 100, 103, 115, 102, 87, 92, 108, 101, 65, 49, 52, 78, 116, 135, 128, 101, 78, 68, 73, 77, 74, 62, 62, 77, 108, 129, 166, 178, 162, 110, 53, 37, 34, 39, 31, 36, 58, 92, 108, 91, 74, 72, 93, 105, 108, 62, 80, 97, 96, 88, 87, 66, 48, 36, 57, 91, 81, 106, 96, 92, 92, 74, 63, 84, 86, 65, 49, 50, 50, 56, 72, 99, 102, 100, 94, 95, 94, 89, 87, 86, 58, 44, 28, 22, 20, 23, 21, 21, 28, 24, 28, 38, 41, 35, 42, 66, 66, 76, 85, 54, 49, 73, 109, 126, 105, 84, 67, 67, 83, 91, 80, 73, 68, 92, 124, 139, 132, 115, 107, 105, 105, 96, 84, 73, 73, 76, 88, 97, 157, 167, 141, 93, 46, 38, 40, 48, 35, 28, 32, 54, 75, 82, 83, 86, 91, 107, 110, 83, 94, 108, 76, 81, 92, 88, 61, 39, 55, 92, 86, 114, 107, 99, 94, 75, 61, 82, 84, 66, 57, 61, 66, 68, 79, 98, 98, 93, 90, 96, 91, 76, 69, 79, 42, 34, 21, 19, 28, 35, 36, 35, 44, 31, 26, 25, 26, 25, 40, 66, 87, 87, 68, 37, 34, 65, 107, 138, 58, 66, 78, 65, 38, 45, 76, 104, 107, 116, 130, 136, 136, 139, 149, 155, 122, 98, 72, 75, 76, 74, 79, 79, 93, 102, 87, 52, 37, 32, 35, 35, 24, 22, 31, 46, 62, 76, 85, 88, 105, 113, 106, 61, 70, 78, 56, 67, 80, 95, 78, 55, 56, 75, 101, 119, 103, 96, 116, 105, 77, 81, 78, 66, 64, 75, 79, 77, 84, 96, 89, 81, 80, 85, 72, 57, 62, 84, 45, 38, 29, 32, 38, 42, 44, 45, 49, 37, 32, 38, 36, 40, 64, 87, 102, 85, 46, 35, 43, 83, 106, 118, 43, 50, 63, 66, 64, 84, 115, 137, 129, 127, 117, 117, 130, 152, 183, 194, 153, 108, 65, 64, 70, 69, 71, 75, 58, 69, 75, 63, 62, 54, 41, 25, 26, 32, 34, 37, 46, 69, 94, 104, 155, 139, 110, 23, 28, 32, 44, 52, 68, 89, 95, 72, 55, 50, 99, 98, 70, 71, 118, 109, 74, 71, 69, 64, 67, 80, 86, 80, 88, 96, 86, 75, 72, 76, 59, 45, 69, 94, 51, 41, 38, 39, 38, 44, 41, 38, 30, 24, 22, 38, 40, 44, 71, 88, 106, 77, 54, 53, 88, 126, 123, 98, 73, 51, 43, 76, 146, 187, 193, 171, 129, 115, 102, 99, 113, 149, 196, 219, 197, 140, 85, 67, 78, 78, 76, 90, 83, 93, 118, 125, 121, 113, 72, 35, 48, 49, 31, 23, 30, 61, 106, 130, 208, 166, 121, 26, 31, 33, 22, 28, 57, 98, 108, 91, 64, 56, 42, 30, 33, 57, 70, 57, 51, 57, 61, 118, 138, 99, 62, 69, 82, 75, 57, 57, 49, 34, 41, 64, 84, 87, 61, 52, 46, 42, 40, 37, 42, 37, 46, 45, 46, 42, 34, 51, 79, 103, 93, 88, 77, 76, 99, 133, 126, 108, 50, 74, 81, 105, 157, 179, 159, 168, 105, 95, 80, 84, 99, 113, 171, 210, 207, 151, 105, 94, 90, 66, 51, 49, 62, 87, 128, 165, 161, 118, 71, 50, 55, 69, 75, 71, 64, 57, 70, 105, 190, 176, 149, 28, 30, 31, 32, 31, 49, 83, 98, 89, 72, 73, 58, 38, 33, 42, 46, 40, 47, 61, 55, 98, 109, 77, 49, 63, 78, 74, 57, 60, 51, 35, 42, 67, 84, 86, 62, 53, 45, 38, 37, 36, 36, 34, 41, 41, 38, 33, 33, 51, 78, 104, 88, 93, 87, 87, 106, 131, 126, 99, 72, 94, 86, 77, 103, 103, 85, 84, 86, 91, 108, 117, 109, 123, 147, 168, 215, 184, 137, 105, 78, 52, 39, 46, 63, 66, 70, 87, 88, 71, 59, 63, 58, 78, 87, 90, 80, 81, 85, 83, 163, 161, 143, 29, 30, 30, 40, 31, 42, 67, 86, 87, 82, 92, 80, 58, 39, 33, 25, 21, 40, 65, 45, 70, 67, 44, 33, 56, 73, 73, 62, 63, 53, 38, 44, 68, 84, 87, 63, 53, 44, 37, 40, 42, 38, 37, 35, 35, 27, 26, 35, 55, 81, 105, 81, 92, 96, 98, 113, 122, 119, 96, 90, 123, 108, 78, 83, 75, 69, 82, 129, 157, 168, 134, 104, 111, 118, 114, 178, 166, 138, 108, 65, 41, 47, 74, 74, 56, 41, 42, 35, 30, 38, 56, 55, 85, 97, 89, 79, 93, 90, 75, 78, 99, 110, 29, 29, 29, 35, 28, 38, 65, 85, 88, 89, 99, 91, 78, 57, 39, 24, 24, 41, 63, 33, 45, 33, 19, 23, 49, 67, 73, 67, 69, 58, 40, 46, 69, 83, 85, 61, 52, 43, 43, 51, 58, 52, 50, 35, 33, 24, 25, 44, 65, 90, 105, 69, 79, 94, 101, 113, 109, 97, 89, 79, 130, 130, 94, 93, 100, 110, 139, 152, 187, 170, 95, 68, 87, 115, 111, 155, 160, 161, 133, 78, 49, 67, 102, 93, 73, 61, 57, 43, 27, 21, 34, 41, 89, 115, 86, 72, 84, 87, 80, 56, 84, 111, 27, 30, 30, 26, 24, 38, 75, 91, 91, 89, 94, 92, 90, 76, 58, 45, 47, 50, 56, 27, 32, 21, 16, 24, 48, 66, 75, 74, 75, 62, 45, 46, 68, 81, 82, 58, 49, 45, 55, 72, 79, 70, 60, 37, 31, 25, 35, 61, 81, 100, 98, 58, 58, 74, 92, 106, 99, 79, 65, 75, 139, 143, 99, 89, 93, 109, 129, 96, 122, 106, 51, 42, 76, 120, 134, 162, 193, 213, 172, 109, 71, 78, 101, 101, 86, 71, 66, 54, 31, 21, 31, 22, 91, 152, 138, 112, 94, 93, 83, 111, 138, 152, 28, 29, 31, 21, 23, 41, 81, 96, 92, 85, 88, 88, 93, 84, 74, 72, 77, 67, 52, 28, 35, 30, 31, 38, 53, 67, 79, 79, 80, 68, 47, 49, 69, 81, 80, 54, 48, 51, 68, 90, 95, 78, 58, 37, 29, 27, 44, 75, 92, 99, 86, 51, 39, 49, 79, 102, 91, 71, 50, 107, 180, 177, 117, 90, 80, 76, 81, 73, 89, 77, 51, 45, 73, 101, 109, 125, 170, 192, 155, 111, 86, 92, 99, 102, 78, 56, 50, 41, 31, 31, 48, 17, 84, 166, 196, 165, 118, 90, 83, 103, 135, 154, 28, 29, 31, 28, 26, 42, 76, 92, 89, 84, 86, 86, 90, 76, 76, 90, 102, 86, 54, 37, 45, 52, 60, 59, 60, 72, 82, 84, 85, 71, 50, 50, 68, 81, 81, 53, 50, 58, 78, 99, 97, 69, 41, 33, 25, 25, 51, 83, 98, 90, 68, 51, 28, 29, 67, 102, 92, 69, 48, 122, 201, 203, 144, 117, 107, 83, 81, 103, 95, 73, 58, 54, 63, 74, 80, 89, 117, 130, 104, 80, 81, 94, 98, 105, 81, 66, 61, 50, 38, 39, 51, 35, 55, 110, 164, 151, 105, 74, 87, 63, 108, 143, 28, 30, 31, 38, 31, 43, 68, 88, 86, 85, 89, 87, 83, 66, 69, 94, 114, 98, 55, 44, 52, 68, 81, 73, 62, 74, 84, 88, 87, 72, 51, 51, 68, 81, 80, 55, 51, 62, 83, 102, 94, 59, 23, 26, 22, 24, 53, 85, 98, 80, 54, 55, 26, 19, 61, 102, 101, 65, 47, 96, 178, 195, 153, 141, 142, 120, 111, 107, 69, 46, 42, 47, 52, 73, 92, 109, 120, 116, 84, 56, 65, 79, 80, 109, 103, 100, 98, 84, 53, 41, 37, 58, 28, 32, 84, 96, 62, 59, 91, 85, 132, 168, 28, 23, 41, 54, 41, 37, 53, 74, 77, 79, 71, 72, 99, 114, 102, 83, 81, 80, 64, 44, 58, 65, 74, 65, 71, 81, 96, 84, 86, 70, 43, 44, 75, 85, 68, 43, 29, 32, 43, 48, 37, 26, 29, 29, 31, 37, 68, 93, 93, 62, 35, 33, 17, 22, 62, 95, 81, 62, 62, 107, 137, 154, 158, 153, 142, 113, 80, 77, 82, 83, 71, 69, 78, 83, 86, 129, 117, 123, 135, 107, 65, 58, 86, 100, 109, 102, 109, 100, 93, 73, 59, 65, 44, 40, 47, 46, 40, 64, 91, 92, 88, 119, 31, 19, 28, 33, 28, 30, 45, 58, 66, 68, 62, 85, 108, 119, 88, 68, 63, 67, 63, 83, 79, 72, 66, 80, 85, 77, 74, 79, 86, 69, 42, 46, 79, 88, 70, 40, 23, 24, 32, 37, 25, 16, 23, 26, 37, 50, 76, 89, 84, 54, 33, 33, 17, 25, 64, 90, 73, 51, 51, 86, 104, 113, 122, 131, 131, 104, 74, 64, 74, 78, 82, 81, 87, 94, 89, 134, 136, 123, 102, 87, 90, 103, 110, 143, 141, 121, 105, 98, 111, 107, 82, 66, 47, 42, 50, 45, 46, 70, 96, 76, 75, 91, 26, 22, 31, 27, 29, 35, 43, 45, 55, 55, 49, 55, 80, 92, 75, 55, 64, 82, 98, 108, 83, 57, 64, 90, 102, 86, 61, 75, 83, 68, 43, 52, 85, 95, 76, 44, 24, 20, 28, 32, 19, 15, 19, 28, 48, 68, 84, 85, 69, 45, 28, 33, 18, 31, 68, 83, 58, 38, 44, 75, 87, 92, 104, 123, 138, 118, 93, 90, 90, 91, 94, 98, 97, 106, 106, 127, 145, 130, 95, 103, 144, 156, 130, 172, 173, 144, 100, 93, 124, 132, 98, 80, 65, 52, 49, 41, 43, 61, 83, 79, 74, 72, 21, 31, 42, 34, 43, 46, 46, 41, 47, 48, 45, 38, 62, 76, 74, 56, 62, 83, 101, 88, 57, 35, 54, 81, 103, 97, 77, 72, 78, 68, 50, 58, 89, 101, 85, 52, 31, 26, 32, 34, 24, 22, 25, 38, 61, 84, 93, 79, 55, 34, 26, 34, 20, 36, 69, 73, 45, 28, 38, 64, 79, 89, 99, 116, 138, 136, 123, 147, 123, 107, 94, 101, 101, 110, 125, 109, 114, 114, 106, 127, 154, 145, 113, 147, 180, 166, 109, 90, 117, 125, 97, 102, 94, 73, 53, 41, 40, 50, 69, 94, 77, 69, 39, 37, 33, 31, 42, 41, 35, 41, 57, 64, 54, 75, 82, 92, 78, 65, 43, 49, 55, 55, 34, 26, 35, 57, 81, 89, 89, 70, 77, 72, 52, 62, 94, 104, 88, 57, 34, 25, 28, 31, 25, 28, 32, 51, 71, 92, 93, 72, 47, 29, 24, 33, 23, 41, 68, 63, 35, 25, 37, 52, 69, 85, 89, 100, 121, 138, 140, 168, 127, 96, 83, 90, 100, 113, 139, 133, 94, 81, 98, 113, 98, 82, 84, 103, 161, 175, 131, 98, 102, 105, 92, 113, 110, 96, 74, 62, 65, 75, 85, 87, 68, 63, 79, 52, 26, 28, 40, 29, 23, 48, 87, 95, 74, 95, 85, 83, 84, 78, 56, 43, 46, 43, 37, 25, 28, 35, 55, 69, 85, 70, 83, 77, 55, 66, 98, 104, 85, 53, 28, 16, 19, 25, 22, 29, 38, 65, 81, 91, 88, 66, 44, 29, 24, 33, 25, 42, 63, 55, 30, 31, 55, 66, 79, 88, 90, 105, 130, 148, 141, 142, 100, 75, 90, 100, 102, 115, 140, 187, 123, 84, 92, 91, 64, 60, 82, 83, 126, 150, 131, 98, 83, 86, 91, 99, 103, 108, 98, 93, 108, 122, 123, 65, 60, 54, 111, 89, 63, 66, 65, 43, 30, 66, 121, 128, 96, 108, 80, 60, 79, 83, 75, 59, 63, 48, 39, 23, 26, 31, 50, 70, 80, 74, 88, 81, 59, 69, 98, 101, 83, 55, 30, 16, 19, 28, 28, 38, 51, 81, 87, 84, 78, 62, 45, 34, 24, 32, 26, 41, 58, 50, 28, 41, 83, 87, 84, 78, 83, 116, 146, 147, 118, 108, 82, 85, 126, 143, 118, 109, 116, 168, 139, 104, 85, 82, 82, 82, 86, 86, 79, 87, 91, 77, 60, 62, 87, 69, 89, 103, 100, 106, 126, 138, 135, 77, 80, 64, 122, 126, 117, 119, 102, 72, 51, 86, 139, 144, 107, 149, 103, 68, 68, 81, 68, 53, 48, 54, 30, 22, 21, 39, 61, 85, 81, 79, 87, 83, 64, 70, 99, 100, 84, 59, 38, 22, 26, 37, 42, 52, 63, 92, 90, 80, 71, 60, 48, 36, 25, 32, 26, 40, 54, 47, 31, 45, 93, 90, 66, 43, 57, 108, 140, 128, 84, 93, 86, 115, 166, 186, 139, 94, 76, 89, 106, 101, 67, 73, 97, 97, 69, 86, 45, 35, 45, 52, 41, 43, 82, 49, 81, 93, 89, 99, 117, 126, 118, 111, 118, 83, 106, 128, 148, 238, 206, 138, 67, 57, 80, 100, 97, 179, 137, 77, 52, 67, 82, 75, 60, 50, 43, 32, 32, 44, 59, 76, 78, 77, 79, 71, 68, 76, 98, 102, 91, 67, 38, 21, 26, 31, 44, 77, 117, 86, 94, 96, 72, 42, 24, 25, 29, 29, 26, 26, 41, 66, 80, 88, 80, 61, 46, 49, 72, 92, 82, 66, 57, 50, 49, 65, 89, 113, 114, 92, 67, 75, 55, 37, 35, 63, 93, 98, 96, 88, 76, 61, 45, 35, 39, 53, 72, 69, 60, 64, 89, 107, 105, 104, 101, 121, 109, 95, 109, 111, 111, 217, 223, 179, 96, 43, 52, 77, 78, 110, 75, 42, 33, 51, 72, 83, 82, 58, 42, 30, 26, 35, 50, 69, 73, 74, 79, 75, 67, 78, 99, 103, 90, 78, 47, 24, 24, 37, 50, 82, 114, 91, 92, 87, 66, 40, 35, 55, 72, 48, 49, 57, 69, 88, 92, 89, 78, 61, 43, 45, 69, 89, 80, 54, 37, 36, 48, 66, 76, 84, 88, 97, 103, 94, 66, 41, 44, 66, 87, 89, 85, 80, 74, 66, 55, 50, 52, 63, 76, 68, 55, 58, 82, 97, 99, 102, 105, 115, 112, 106, 106, 110, 105, 174, 235, 227, 132, 39, 34, 59, 64, 50, 32, 24, 26, 40, 62, 87, 97, 68, 49, 29, 20, 26, 44, 66, 73, 73, 79, 80, 68, 82, 98, 103, 89, 83, 55, 29, 29, 48, 65, 88, 108, 92, 88, 76, 56, 35, 42, 81, 109, 66, 71, 86, 96, 103, 94, 76, 59, 45, 33, 38, 70, 90, 79, 50, 27, 32, 55, 77, 68, 52, 59, 100, 140, 121, 92, 64, 62, 71, 82, 79, 72, 70, 76, 79, 74, 72, 70, 74, 81, 67, 51, 50, 69, 80, 84, 93, 102, 100, 103, 109, 87, 113, 129, 124, 216, 237, 146, 56, 50, 62, 66, 40, 41, 42, 36, 40, 56, 78, 95, 76, 60, 33, 20, 30, 51, 71, 83, 72, 79, 82, 72, 86, 101, 104, 89, 72, 55, 39, 44, 67, 82, 92, 92, 89, 83, 70, 49, 30, 40, 77, 106, 72, 83, 97, 105, 99, 79, 53, 32, 27, 27, 50, 84, 95, 73, 55, 44, 45, 68, 87, 68, 38, 44, 88, 137, 121, 102, 80, 72, 76, 82, 79, 72, 72, 85, 96, 95, 88, 82, 78, 78, 66, 52, 44, 54, 58, 61, 72, 85, 89, 91, 102, 64, 85, 113, 93, 169, 198, 129, 73, 73, 79, 72, 49, 57, 60, 48, 40, 48, 68, 89, 81, 69, 45, 33, 46, 66, 81, 92, 72, 82, 81, 77, 89, 103, 102, 89, 61, 58, 65, 69, 84, 96, 92, 79, 81, 80, 69, 45, 25, 27, 52, 77, 82, 93, 101, 104, 87, 63, 42, 29, 31, 48, 83, 106, 96, 61, 55, 56, 60, 69, 74, 59, 40, 39, 58, 84, 77, 67, 62, 61, 75, 84, 89, 86, 83, 97, 106, 103, 94, 83, 76, 76, 68, 58, 47, 48, 44, 42, 51, 70, 101, 98, 102, 70, 62, 81, 110, 139, 143, 111, 83, 87, 89, 81, 49, 53, 54, 45, 32, 36, 62, 83, 86, 76, 61, 59, 66, 77, 83, 88, 72, 82, 81, 82, 92, 106, 103, 90, 74, 86, 103, 98, 96, 98, 88, 75, 75, 79, 70, 46, 26, 24, 41, 66, 97, 103, 103, 96, 75, 57, 46, 43, 55, 77, 108, 122, 99, 61, 51, 56, 62, 56, 45, 40, 43, 42, 35, 32, 29, 27, 33, 44, 71, 86, 97, 99, 97, 99, 103, 99, 92, 81, 75, 78, 72, 64, 53, 52, 45, 39, 43, 69, 113, 122, 117, 122, 104, 112, 168, 152, 129, 114, 98, 93, 93, 96, 55, 53, 46, 38, 27, 32, 58, 76, 95, 86, 79, 79, 79, 79, 71, 66, 71, 82, 84, 84, 94, 107, 102, 89, 115, 134, 142, 119, 96, 92, 87, 79, 74, 77, 69, 46, 28, 31, 55, 83, 106, 104, 94, 75, 55, 50, 54, 64, 73, 78, 96, 116, 112, 85, 63, 58, 55, 43, 31, 34, 50, 55, 45, 32, 31, 33, 41, 53, 73, 85, 93, 94, 95, 97, 94, 88, 82, 78, 82, 88, 75, 66, 63, 67, 58, 49, 60, 81, 99, 126, 136, 177, 173, 181, 222, 184, 143, 129, 113, 97, 94, 113, 72, 68, 48, 36, 27, 32, 52, 66, 101, 95, 90, 90, 86, 77, 59, 45, 72, 80, 86, 84, 95, 107, 104, 88, 150, 172, 170, 129, 94, 85, 86, 86, 76, 75, 67, 48, 32, 42, 76, 105, 107, 97, 81, 52, 37, 41, 55, 73, 75, 59, 68, 103, 125, 112, 86, 67, 51, 39, 34, 41, 57, 69, 68, 62, 64, 69, 71, 75, 80, 83, 83, 82, 87, 92, 88, 79, 73, 77, 89, 96, 76, 65, 69, 79, 69, 61, 79, 94, 74, 118, 146, 133, 190, 208, 199, 144, 93, 76, 98, 122, 115, 105, 88, 77, 49, 37, 32, 39, 48, 58, 75, 76, 78, 94, 92, 79, 60, 35, 59, 62, 89, 109, 107, 96, 90, 102, 200, 186, 165, 129, 102, 83, 82, 84, 74, 57, 44, 41, 62, 81, 94, 97, 97, 79, 50, 43, 48, 62, 75, 77, 60, 58, 58, 92, 137, 167, 137, 88, 45, 42, 51, 60, 73, 82, 83, 77, 81, 86, 80, 67, 57, 63, 74, 82, 88, 103, 105, 86, 86, 97, 96, 75, 70, 101, 87, 69, 86, 79, 50, 51, 86, 127, 174, 59, 91, 103, 112, 84, 52, 52, 84, 111, 116, 106, 99, 89, 75, 59, 47, 47, 50, 55, 75, 80, 87, 87, 80, 64, 46, 36, 70, 66, 78, 97, 108, 98, 91, 95, 115, 119, 113, 108, 90, 80, 78, 77, 59, 50, 44, 61, 78, 101, 109, 104, 88, 68, 52, 58, 68, 76, 71, 64, 103, 104, 107, 106, 123, 129, 99, 59, 76, 64, 64, 73, 78, 88, 93, 92, 81, 75, 55, 40, 48, 81, 87, 76, 64, 73, 73, 69, 84, 103, 108, 87, 76, 104, 98, 83, 95, 88, 59, 56, 72, 123, 185, 32, 44, 41, 42, 27, 27, 35, 62, 88, 104, 103, 111, 105, 103, 85, 73, 63, 56, 52, 60, 71, 89, 91, 86, 78, 78, 81, 80, 69, 66, 94, 103, 98, 92, 88, 49, 64, 77, 92, 83, 77, 72, 68, 61, 58, 59, 82, 97, 112, 108, 99, 71, 62, 61, 72, 85, 83, 62, 42, 105, 119, 131, 110, 96, 83, 71, 57, 94, 87, 80, 83, 84, 89, 92, 87, 64, 56, 31, 27, 74, 146, 165, 133, 91, 85, 78, 70, 79, 98, 100, 89, 86, 118, 107, 82, 91, 79, 57, 61, 58, 99, 155, 35, 44, 39, 32, 24, 32, 34, 44, 62, 81, 94, 112, 113, 113, 105, 98, 85, 69, 62, 49, 69, 91, 88, 79, 75, 93, 113, 95, 73, 67, 90, 104, 101, 92, 87, 55, 67, 82, 89, 84, 76, 71, 67, 90, 83, 83, 92, 104, 106, 90, 78, 60, 70, 78, 79, 77, 64, 49, 39, 73, 106, 131, 116, 87, 79, 92, 102, 94, 94, 93, 89, 95, 88, 77, 68, 48, 57, 34, 23, 78, 164, 196, 164, 114, 109, 97, 84, 79, 97, 117, 131, 138, 159, 131, 91, 92, 79, 61, 73, 53, 64, 87, 34, 42, 37, 40, 42, 37, 27, 31, 40, 61, 82, 99, 103, 107, 111, 109, 102, 89, 84, 53, 76, 92, 83, 66, 67, 93, 122, 110, 86, 80, 92, 113, 105, 92, 87, 86, 87, 91, 85, 79, 78, 79, 82, 112, 102, 93, 95, 103, 98, 83, 71, 61, 79, 94, 79, 57, 37, 48, 62, 63, 104, 131, 124, 99, 103, 114, 127, 95, 96, 105, 105, 112, 94, 72, 57, 47, 66, 49, 20, 40, 102, 130, 119, 93, 100, 92, 86, 84, 103, 152, 191, 193, 191, 148, 103, 104, 99, 80, 86, 59, 51, 51, 94, 92, 54, 36, 43, 34, 22, 19, 27, 48, 68, 82, 85, 93, 103, 102, 105, 104, 99, 60, 76, 87, 96, 93, 114, 152, 186, 120, 101, 96, 117, 120, 110, 95, 93, 98, 90, 81, 75, 66, 77, 88, 104, 109, 105, 96, 105, 109, 108, 94, 79, 73, 84, 96, 78, 52, 41, 64, 84, 76, 103, 117, 108, 109, 109, 101, 97, 99, 101, 112, 119, 116, 102, 83, 59, 48, 55, 57, 40, 39, 69, 95, 109, 95, 98, 90, 87, 79, 85, 119, 160, 165, 158, 113, 83, 107, 116, 93, 90, 80, 69, 67, 187, 180, 96, 35, 37, 35, 30, 23, 28, 35, 45, 70, 72, 77, 82, 87, 97, 108, 106, 77, 83, 93, 104, 118, 148, 191, 223, 121, 115, 118, 143, 128, 112, 101, 101, 99, 91, 75, 69, 62, 75, 95, 115, 104, 104, 109, 122, 130, 124, 103, 80, 87, 83, 84, 76, 73, 77, 92, 97, 93, 95, 100, 105, 133, 137, 112, 84, 103, 99, 102, 106, 102, 98, 84, 67, 68, 54, 65, 83, 82, 83, 101, 129, 113, 95, 85, 92, 85, 72, 66, 77, 85, 89, 61, 63, 124, 148, 122, 111, 117, 101, 93, 222, 219, 118, 42, 43, 44, 43, 37, 32, 24, 27, 69, 66, 66, 64, 78, 88, 109, 113, 106, 110, 106, 94, 96, 117, 151, 180, 122, 124, 139, 144, 139, 113, 105, 107, 105, 98, 83, 66, 66, 74, 95, 113, 105, 110, 127, 138, 150, 135, 99, 71, 92, 80, 69, 76, 96, 110, 115, 110, 110, 102, 102, 128, 173, 197, 164, 107, 100, 93, 85, 79, 87, 88, 78, 77, 104, 78, 83, 108, 104, 78, 78, 111, 98, 70, 65, 94, 111, 90, 59, 44, 42, 50, 48, 77, 162, 199, 166, 145, 155, 121, 96, 124, 118, 91, 82, 76, 66, 41, 32, 26, 30, 37, 44, 62, 84, 88, 72, 71, 81, 100, 115, 126, 113, 110, 121, 101, 85, 104, 82, 100, 126, 125, 117, 103, 108, 116, 103, 79, 53, 45, 66, 93, 108, 117, 122, 123, 114, 104, 93, 94, 97, 101, 103, 89, 83, 80, 96, 108, 110, 115, 96, 73, 45, 98, 190, 192, 138, 99, 72, 67, 63, 61, 66, 78, 86, 93, 100, 97, 95, 86, 82, 78, 79, 75, 85, 78, 74, 71, 65, 67, 58, 53, 62, 57, 56, 61, 77, 99, 119, 131, 137, 129, 119, 72, 88, 109, 105, 94, 84, 63, 47, 36, 32, 33, 43, 63, 84, 85, 71, 69, 83, 99, 103, 118, 101, 102, 120, 106, 83, 98, 61, 99, 122, 118, 99, 99, 108, 108, 93, 71, 51, 50, 73, 98, 108, 110, 109, 105, 97, 91, 86, 85, 94, 99, 88, 96, 97, 107, 120, 110, 85, 69, 48, 62, 63, 86, 126, 117, 63, 48, 42, 51, 64, 76, 89, 88, 88, 85, 82, 77, 72, 74, 66, 69, 63, 72, 81, 84, 78, 78, 82, 77, 69, 65, 53, 53, 43, 50, 56, 75, 97, 111, 132, 123, 119, 123, 99, 111, 122, 114, 105, 92, 76, 60, 47, 41, 50, 70, 85, 85, 76, 71, 86, 99, 110, 123, 100, 95, 117, 113, 87, 95, 64, 126, 148, 116, 86, 98, 106, 93, 73, 62, 53, 62, 82, 99, 107, 108, 110, 107, 97, 92, 89, 89, 102, 102, 103, 107, 108, 114, 106, 83, 52, 40, 87, 160, 172, 145, 140, 113, 75, 58, 54, 55, 72, 78, 75, 72, 67, 59, 79, 72, 69, 70, 66, 68, 65, 74, 60, 62, 68, 70, 66, 60, 53, 53, 63, 67, 67, 69, 78, 93, 110, 117, 128, 114, 112, 175, 114, 87, 104, 99, 98, 98, 96, 88, 75, 63, 67, 80, 93, 92, 85, 79, 92, 101, 151, 157, 118, 95, 116, 118, 97, 103, 99, 170, 188, 128, 88, 101, 103, 75, 54, 54, 58, 75, 91, 103, 106, 105, 110, 107, 98, 96, 97, 105, 107, 103, 125, 111, 94, 79, 58, 46, 44, 37, 112, 194, 198, 138, 103, 90, 73, 68, 89, 85, 77, 70, 59, 45, 36, 31, 94, 88, 87, 76, 77, 74, 74, 71, 54, 53, 59, 55, 43, 41, 43, 46, 67, 76, 93, 105, 121, 131, 136, 133, 116, 106, 98, 110, 78, 71, 62, 56, 64, 76, 87, 96, 92, 84, 82, 90, 100, 100, 93, 90, 96, 102, 190, 194, 145, 101, 112, 120, 110, 115, 115, 181, 194, 133, 93, 104, 98, 65, 46, 52, 67, 88, 100, 108, 106, 98, 94, 90, 89, 102, 116, 128, 124, 122, 99, 86, 64, 46, 38, 42, 55, 58, 98, 152, 136, 78, 64, 77, 79, 87, 85, 89, 86, 88, 77, 59, 40, 32, 96, 94, 87, 70, 68, 62, 55, 54, 71, 72, 64, 52, 43, 44, 57, 69, 66, 79, 97, 123, 132, 128, 120, 114, 97, 95, 87, 60, 59, 83, 49, 40, 40, 51, 72, 88, 94, 92, 90, 95, 105, 102, 96, 94, 96, 100, 178, 197, 156, 105, 107, 118, 113, 121, 109, 153, 161, 126, 94, 102, 89, 64, 49, 56, 76, 97, 109, 110, 110, 101, 96, 93, 100, 130, 155, 166, 162, 156, 52, 57, 60, 58, 58, 66, 72, 92, 85, 113, 86, 54, 80, 101, 95, 103, 60, 71, 98, 117, 112, 98, 76, 58, 82, 77, 63, 52, 42, 34, 27, 28, 62, 71, 64, 57, 51, 53, 70, 86, 93, 106, 120, 128, 125, 112, 94, 84, 84, 83, 81, 93, 91, 104, 93, 74, 57, 55, 72, 90, 99, 96, 90, 94, 101, 99, 93, 92, 91, 92, 112, 159, 144, 101, 104, 114, 110, 119, 116, 130, 133, 119, 100, 93, 75, 57, 51, 63, 86, 103, 114, 110, 113, 109, 109, 113, 123, 149, 172, 172, 157, 138, 67, 72, 83, 81, 76, 82, 90, 105, 83, 89, 73, 72, 97, 97, 62, 64, 81, 87, 96, 104, 108, 96, 88, 78, 72, 64, 53, 47, 34, 28, 23, 24, 40, 58, 72, 77, 72, 75, 90, 105, 128, 130, 131, 117, 112, 103, 88, 81, 78, 77, 73, 120, 122, 114, 148, 121, 86, 77, 84, 100, 106, 99, 91, 92, 96, 93, 90, 87, 86, 85, 50, 117, 125, 95, 100, 110, 106, 115, 138, 133, 131, 118, 110, 85, 60, 50, 52, 69, 96, 108, 115, 113, 112, 107, 110, 116, 123, 138, 150, 140, 105, 78, 121, 117, 105, 86, 76, 77, 89, 105, 139, 145, 130, 147, 169, 121, 70, 55, 138, 120, 96, 70, 55, 61, 73, 78, 77, 66, 64, 54, 46, 42, 34, 37, 37, 63, 94, 110, 111, 112, 123, 137, 131, 122, 108, 100, 90, 90, 86, 93, 73, 72, 66, 94, 96, 122, 114, 117, 116, 117, 102, 98, 96, 102, 90, 94, 97, 107, 106, 102, 91, 86, 100, 105, 115, 117, 116, 104, 110, 118, 133, 138, 138, 111, 90, 81, 68, 57, 81, 93, 117, 127, 125, 124, 120, 121, 89, 81, 81, 86, 85, 86, 81, 76, 97, 92, 70, 60, 70, 85, 104, 124, 157, 162, 165, 166, 152, 123, 86, 54, 105, 89, 75, 74, 73, 64, 70, 83, 80, 66, 53, 53, 53, 41, 35, 38, 29, 41, 54, 87, 119, 128, 123, 104, 83, 92, 103, 109, 111, 98, 81, 72, 59, 29, 35, 112, 97, 63, 90, 99, 99, 101, 94, 99, 103, 114, 118, 114, 107, 109, 110, 110, 112, 107, 84, 84, 86, 103, 109, 119, 120, 131, 139, 157, 150, 131, 103, 90, 67, 50, 66, 91, 123, 139, 133, 113, 96, 96, 92, 82, 77, 71, 79, 85, 85, 85, 70, 72, 79, 80, 82, 77, 70, 64, 79, 79, 82, 92, 92, 79, 64, 51, 91, 75, 65, 65, 59, 56, 81, 105, 84, 62, 57, 62, 57, 42, 32, 36, 33, 50, 87, 112, 121, 114, 105, 93, 101, 105, 104, 101, 93, 78, 66, 55, 53, 33, 38, 107, 99, 46, 72, 80, 76, 77, 73, 78, 88, 98, 124, 120, 113, 110, 112, 113, 125, 120, 109, 102, 92, 83, 87, 93, 93, 94, 132, 149, 154, 134, 112, 98, 76, 59, 90, 119, 146, 147, 133, 108, 94, 94, 96, 83, 72, 64, 75, 81, 87, 88, 83, 80, 89, 85, 73, 56, 40, 29, 37, 35, 36, 50, 57, 63, 61, 57, 83, 81, 90, 98, 82, 69, 86, 108, 72, 50, 46, 55, 55, 42, 35, 41, 73, 88, 111, 112, 98, 86, 86, 89, 117, 113, 96, 86, 68, 54, 50, 40, 46, 42, 44, 82, 90, 69, 72, 69, 63, 55, 47, 42, 52, 58, 94, 107, 119, 127, 125, 120, 123, 119, 125, 127, 119, 83, 68, 63, 70, 69, 115, 127, 135, 120, 105, 102, 97, 90, 130, 141, 149, 129, 110, 95, 94, 101, 90, 80, 71, 73, 76, 83, 88, 82, 122, 110, 87, 65, 46, 36, 35, 36, 52, 49, 48, 55, 60, 67, 59, 54, 72, 98, 130, 149, 124, 89, 75, 85, 49, 34, 27, 38, 51, 48, 55, 64, 129, 124, 103, 81, 67, 68, 86, 101, 113, 98, 79, 64, 50, 42, 45, 43, 40, 48, 52, 110, 88, 71, 68, 69, 63, 52, 38, 32, 39, 40, 68, 100, 136, 164, 160, 144, 126, 114, 108, 136, 140, 111, 77, 79, 95, 101, 121, 129, 121, 111, 97, 109, 115, 115, 131, 132, 130, 117, 99, 83, 79, 80, 73, 73, 73, 82, 87, 86, 83, 72, 113, 97, 69, 49, 37, 42, 49, 55, 62, 55, 54, 57, 61, 54, 45, 39, 49, 80, 119, 134, 111, 77, 62, 71, 40, 27, 23, 41, 62, 71, 89, 104, 125, 110, 78, 68, 71, 84, 96, 103, 83, 72, 60, 46, 43, 45, 51, 60, 37, 48, 57, 185, 106, 45, 55, 64, 66, 61, 51, 48, 52, 52, 64, 103, 145, 177, 173, 147, 118, 106, 116, 152, 168, 140, 111, 113, 120, 120, 141, 149, 133, 110, 104, 117, 126, 130, 120, 129, 130, 144, 133, 114, 83, 59, 66, 66, 75, 81, 96, 89, 79, 67, 59, 52, 53, 54, 55, 60, 64, 64, 63, 55, 53, 52, 54, 52, 45, 40, 32, 40, 57, 68, 52, 38, 51, 75, 49, 35, 41, 68, 88, 99, 111, 124, 78, 69, 68, 84, 100, 106, 95, 83, 51, 49, 48, 40, 45, 52, 55, 66, 43, 41, 51, 168, 94, 33, 40, 51, 61, 63, 59, 56, 55, 54, 58, 88, 122, 134, 123, 99, 83, 76, 104, 136, 157, 138, 145, 156, 149, 120, 129, 141, 137, 110, 112, 123, 137, 139, 132, 132, 133, 155, 163, 149, 106, 66, 69, 65, 70, 75, 92, 88, 82, 73, 50, 49, 58, 63, 67, 69, 72, 74, 76, 63, 53, 43, 48, 54, 52, 51, 33, 23, 24, 30, 27, 21, 39, 63, 52, 50, 67, 97, 113, 104, 97, 96, 64, 69, 85, 95, 97, 86, 69, 54, 40, 41, 46, 44, 49, 52, 51, 52, 50, 31, 41, 66, 47, 38, 31, 37, 53, 58, 54, 48, 40, 39, 45, 63, 83, 77, 59, 42, 42, 44, 46, 80, 103, 124, 163, 202, 190, 144, 100, 113, 123, 115, 114, 130, 143, 141, 147, 125, 115, 127, 152, 147, 114, 84, 73, 69, 67, 71, 76, 84, 86, 79, 84, 82, 72, 63, 66, 69, 74, 81, 87, 69, 45, 27, 34, 40, 44, 53, 43, 32, 34, 42, 42, 31, 30, 41, 47, 59, 80, 109, 123, 95, 71, 55, 89, 100, 106, 89, 68, 45, 38, 35, 42, 40, 49, 51, 50, 49, 44, 36, 54, 24, 32, 39, 38, 31, 36, 26, 22, 28, 46, 58, 51, 31, 37, 42, 44, 45, 50, 53, 53, 57, 68, 77, 82, 82, 114, 140, 148, 138, 126, 108, 117, 147, 133, 93, 124, 186, 180, 156, 127, 117, 124, 127, 122, 112, 86, 79, 78, 76, 76, 80, 85, 93, 90, 92, 90, 83, 81, 76, 74, 73, 83, 82, 78, 66, 60, 54, 48, 48, 76, 85, 91, 63, 28, 19, 45, 75, 101, 101, 96, 96, 73, 58, 70, 92, 94, 89, 72, 60, 44, 35, 34, 36, 53, 49, 49, 58, 62, 58, 41, 24, 28, 51, 54, 44, 42, 35, 29, 38, 52, 60, 62, 52, 50, 50, 48, 50, 49, 47, 51, 57, 62, 65, 50, 60, 65, 75, 97, 129, 139, 133, 134, 131, 156, 183, 149, 104, 130, 196, 225, 196, 149, 118, 112, 116, 118, 114, 101, 94, 97, 91, 90, 97, 98, 100, 86, 86, 81, 89, 86, 98, 102, 107, 92, 89, 79, 83, 78, 82, 85, 92, 78, 99, 107, 95, 73, 63, 78, 93, 110, 105, 99, 97, 84, 76, 98, 124, 140, 107, 61, 41, 40, 46, 43, 37, 52, 48, 48, 51, 57, 51, 36, 23, 28, 51, 52, 52, 47, 48, 38, 69, 101, 105, 86, 66, 69, 79, 67, 63, 60, 61, 62, 68, 75, 79, 58, 63, 71, 78, 91, 122, 128, 127, 149, 160, 189, 200, 150, 100, 112, 169, 226, 199, 158, 130, 119, 120, 120, 114, 112, 108, 111, 100, 100, 100, 100, 101, 128, 118, 103, 88, 84, 88, 92, 98, 100, 94, 92, 90, 92, 99, 105, 108, 100, 102, 98, 91, 83, 82, 90, 99, 104, 95, 89, 86, 83, 80, 112, 143, 175, 121, 50, 29, 41, 62, 58, 44, 51, 45, 45, 44, 49, 40, 31, 22, 29, 48, 50, 66, 59, 65, 45, 78, 108, 108, 85, 72, 74, 86, 81, 78, 81, 85, 85, 85, 85, 83, 87, 92, 101, 96, 106, 126, 130, 125, 155, 165, 174, 167, 123, 78, 79, 109, 150, 137, 132, 130, 134, 129, 127, 117, 113, 110, 108, 99, 99, 94, 91, 94, 151, 135, 112, 81, 72, 67, 73, 75, 88, 88, 95, 91, 93, 92, 92, 87, 137, 112, 88, 76, 78, 90, 97, 105, 96, 89, 85, 86, 85, 84, 106, 132, 156, 107, 54, 38, 58, 78, 73, 59, 46, 46, 41, 38, 40, 34, 28, 24, 34, 43, 43, 87, 73, 75, 50, 60, 67, 69, 60, 58, 66, 73, 88, 90, 99, 106, 101, 94, 85, 77, 91, 105, 119, 115, 124, 137, 141, 129, 137, 136, 126, 113, 94, 74, 67, 73, 81, 75, 85, 92, 110, 120, 127, 127, 117, 113, 110, 108, 103, 99, 93, 93, 123, 113, 92, 75, 62, 62, 68, 71, 58, 67, 81, 98, 96, 91, 85, 82, 141, 123, 102, 101, 114, 125, 122, 117, 87, 82, 83, 90, 88, 80, 84, 96, 93, 75, 63, 66, 80, 84, 84, 73, 44, 49, 42, 38, 35, 33, 34, 35, 44, 45, 42, 97, 80, 86, 72, 64, 54, 59, 64, 73, 84, 84, 98, 98, 103, 106, 98, 87, 73, 64, 69, 97, 114, 125, 127, 134, 138, 121, 98, 94, 86, 79, 81, 92, 84, 76, 67, 56, 50, 44, 61, 87, 117, 134, 124, 123, 121, 125, 115, 111, 106, 102, 120, 111, 94, 75, 62, 50, 49, 44, 36, 46, 69, 93, 105, 108, 101, 100, 98, 105, 104, 121, 141, 137, 120, 97, 63, 53, 54, 66, 60, 46, 40, 49, 37, 49, 67, 78, 84, 78, 80, 75, 45, 50, 44, 43, 35, 37, 45, 53, 63, 55, 49, 96, 77, 78, 91, 88, 81, 84, 92, 103, 115, 115, 109, 99, 93, 84, 75, 66, 59, 57, 66, 100, 122, 132, 124, 121, 108, 92, 64, 73, 83, 78, 77, 95, 91, 75, 70, 58, 48, 37, 45, 69, 103, 122, 128, 129, 130, 133, 125, 121, 116, 109, 121, 114, 106, 85, 73, 52, 40, 29, 39, 45, 64, 78, 98, 110, 110, 114, 92, 91, 90, 97, 109, 100, 88, 67, 56, 41, 33, 41, 36, 26, 25, 34, 29, 44, 55, 66, 67, 66, 68, 66, 48, 50, 52, 44, 40, 45, 60, 73, 86, 72, 62, 95, 67, 54, 86, 96, 98, 98, 102, 108, 126, 137, 112, 99, 81, 60, 49, 50, 54, 59, 83, 117, 144, 137, 121, 107, 77, 65, 50, 76, 96, 84, 75, 82, 79, 58, 62, 57, 58, 58, 61, 73, 93, 105, 125, 129, 128, 127, 128, 120, 117, 113, 85, 89, 91, 97, 90, 79, 64, 56, 52, 59, 57, 69, 79, 93, 103, 117, 137, 109, 87, 69, 60, 62, 62, 61, 83, 60, 46, 46, 45, 41, 41, 50, 52, 51, 43, 49, 50, 55, 59, 57, 50, 49, 57, 45, 45, 50, 69, 86, 102, 85, 75, 101, 98, 71, 103, 98, 89, 94, 107, 133, 166, 183, 136, 78, 45, 50, 62, 54, 60, 73, 110, 130, 144, 133, 109, 71, 57, 48, 42, 66, 88, 88, 79, 77, 72, 63, 53, 59, 57, 50, 41, 50, 81, 103, 114, 116, 127, 125, 116, 119, 131, 143, 131, 91, 104, 173, 201, 144, 82, 65, 64, 68, 67, 63, 64, 78, 94, 122, 97, 87, 83, 83, 87, 87, 79, 75, 86, 83, 82, 80, 74, 71, 70, 70, 64, 62, 62, 73, 69, 62, 73, 88, 72, 66, 49, 40, 68, 83, 73, 51, 64, 63, 66, 97, 101, 96, 107, 102, 96, 113, 140, 163, 170, 171, 93, 52, 36, 55, 66, 66, 85, 113, 133, 136, 138, 122, 101, 77, 59, 48, 42, 70, 94, 93, 87, 83, 79, 68, 57, 66, 70, 66, 55, 54, 70, 85, 99, 109, 125, 129, 129, 121, 117, 117, 133, 108, 106, 135, 139, 102, 64, 59, 64, 63, 64, 65, 64, 72, 82, 94, 93, 85, 81, 80, 83, 85, 79, 73, 86, 79, 75, 70, 68, 71, 73, 76, 81, 67, 59, 59, 57, 60, 84, 107, 68, 61, 55, 64, 87, 110, 90, 53, 60, 51, 52, 83, 89, 89, 54, 39, 44, 60, 81, 97, 86, 73, 59, 35, 41, 58, 84, 87, 106, 140, 141, 131, 120, 96, 73, 54, 40, 38, 42, 77, 102, 102, 97, 95, 91, 78, 62, 75, 85, 84, 71, 59, 61, 65, 83, 96, 115, 129, 141, 126, 105, 88, 118, 123, 123, 120, 108, 89, 59, 43, 44, 36, 35, 38, 40, 42, 46, 38, 55, 49, 43, 45, 46, 52, 47, 41, 54, 49, 44, 40, 45, 47, 57, 57, 54, 40, 42, 47, 47, 48, 64, 87, 53, 43, 43, 57, 83, 105, 86, 45, 59, 47, 45, 56, 53, 63, 46, 44, 44, 51, 63, 61, 53, 43, 54, 52, 59, 78, 88, 99, 114, 145, 138, 139, 129, 95, 60, 34, 29, 35, 42, 79, 108, 110, 105, 105, 100, 86, 70, 79, 89, 89, 79, 67, 64, 63, 76, 85, 101, 120, 140, 131, 110, 85, 94, 116, 128, 121, 121, 116, 87, 46, 37, 26, 16, 16, 24, 25, 25, 13, 23, 17, 13, 14, 17, 21, 22, 16, 23, 26, 30, 34, 38, 37, 42, 36, 16, 16, 38, 68, 64, 49, 48, 54, 71, 50, 36, 30, 61, 82, 77, 49, 71, 58, 50, 41, 33, 36, 43, 55, 61, 60, 57, 56, 63, 71, 66, 67, 82, 101, 107, 112, 122, 142, 130, 150, 157, 120, 68, 34, 35, 41, 43, 76, 109, 115, 112, 111, 105, 92, 75, 80, 82, 83, 77, 74, 75, 75, 82, 80, 89, 105, 126, 132, 124, 108, 83, 99, 101, 91, 114, 137, 126, 82, 64, 50, 29, 21, 30, 35, 35, 30, 26, 20, 20, 27, 32, 33, 33, 26, 21, 33, 43, 57, 58, 51, 43, 33, 19, 27, 50, 80, 81, 61, 55, 61, 112, 88, 51, 26, 43, 69, 77, 64, 85, 66, 52, 57, 48, 32, 26, 37, 55, 57, 42, 53, 74, 103, 75, 70, 96, 110, 127, 127, 129, 127, 102, 121, 137, 109, 62, 26, 30, 37, 36, 70, 105, 117, 112, 113, 103, 92, 79, 79, 75, 76, 76, 79, 81, 82, 86, 81, 84, 91, 106, 119, 126, 129, 96, 89, 63, 48, 71, 114, 133, 112, 90, 73, 48, 30, 28, 33, 34, 39, 26, 26, 33, 44, 51, 51, 45, 33, 26, 36, 47, 59, 55, 46, 32, 22, 32, 33, 41, 51, 56, 47, 54, 71, 110, 100, 62, 35, 31, 56, 72, 72, 82, 59, 43, 83, 64, 57, 62, 71, 83, 92, 91, 94, 113, 131, 82, 87, 96, 117, 123, 131, 115, 94, 68, 71, 73, 67, 48, 28, 26, 28, 28, 62, 101, 112, 109, 110, 101, 90, 78, 79, 75, 77, 78, 78, 75, 74, 86, 85, 86, 83, 87, 98, 111, 127, 111, 90, 55, 40, 44, 69, 97, 111, 102, 93, 74, 52, 33, 25, 24, 26, 19, 23, 39, 55, 63, 59, 45, 31, 40, 38, 43, 41, 37, 28, 22, 16, 27, 25, 32, 38, 41, 33, 38, 56, 68, 85, 79, 63, 52, 63, 75, 82, 74, 52, 40, 96, 79, 79, 74, 78, 82, 88, 92, 95, 100, 86, 102, 107, 103, 110, 128, 122, 94, 68, 56, 52, 36, 38, 47, 54, 40, 31, 26, 59, 97, 109, 108, 106, 99, 89, 76, 78, 77, 81, 84, 77, 67, 63, 80, 89, 91, 81, 77, 81, 96, 112, 114, 93, 68, 53, 41, 39, 60, 98, 116, 125, 114, 87, 55, 31, 27, 24, 22, 30, 48, 67, 78, 72, 53, 39, 66, 57, 54, 40, 35, 29, 33, 30, 19, 25, 44, 68, 62, 44, 37, 43, 42, 79, 111, 102, 96, 96, 100, 98, 76, 52, 42, 72, 80, 85, 75, 80, 94, 93, 92, 92, 92, 94, 97, 97, 115, 131, 124, 87, 44, 37, 41, 34, 28, 30, 35, 38, 38, 33, 45, 65, 97, 112, 119, 125, 121, 120, 87, 87, 80, 85, 90, 93, 90, 74, 76, 75, 91, 99, 92, 75, 74, 92, 110, 114, 107, 84, 50, 37, 33, 46, 87, 107, 114, 97, 96, 105, 70, 26, 24, 24, 54, 97, 92, 55, 52, 74, 58, 44, 25, 18, 25, 30, 31, 26, 33, 35, 45, 49, 45, 44, 42, 44, 44, 101, 84, 101, 173, 138, 66, 103, 61, 53, 74, 83, 85, 99, 162, 142, 110, 86, 86, 91, 100, 111, 106, 116, 137, 147, 121, 78, 42, 36, 39, 33, 33, 37, 45, 50, 49, 44, 47, 69, 95, 114, 116, 117, 115, 111, 83, 79, 80, 84, 87, 90, 96, 101, 108, 104, 107, 105, 96, 82, 84, 96, 108, 109, 106, 88, 59, 38, 34, 41, 21, 64, 96, 106, 116, 121, 88, 39, 37, 28, 44, 78, 93, 65, 56, 68, 52, 42, 24, 17, 24, 28, 27, 23, 35, 38, 48, 50, 45, 41, 42, 42, 121, 140, 80, 58, 131, 120, 58, 76, 59, 51, 65, 78, 78, 112, 199, 168, 121, 90, 90, 103, 118, 126, 107, 115, 128, 129, 87, 43, 29, 33, 36, 33, 37, 43, 53, 60, 58, 54, 53, 72, 92, 116, 114, 106, 100, 97, 87, 88, 92, 86, 86, 87, 90, 102, 98, 87, 80, 78, 74, 61, 69, 86, 97, 104, 112, 102, 80, 58, 44, 46, 128, 134, 116, 80, 70, 90, 107, 104, 70, 46, 38, 71, 81, 68, 52, 56, 42, 34, 24, 19, 24, 27, 26, 25, 37, 44, 50, 50, 44, 39, 40, 43, 150, 184, 128, 68, 100, 94, 56, 68, 55, 44, 52, 58, 66, 118, 147, 127, 110, 94, 89, 105, 124, 133, 114, 104, 97, 89, 54, 29, 27, 38, 38, 36, 42, 45, 55, 61, 55, 52, 57, 72, 93, 116, 112, 94, 79, 85, 90, 102, 106, 98, 93, 83, 74, 75, 55, 41, 41, 48, 51, 42, 51, 67, 82, 95, 113, 114, 102, 86, 70, 65, 228, 196, 139, 62, 30, 52, 99, 122, 101, 76, 59, 75, 63, 48, 36, 40, 27, 24, 23, 25, 28, 32, 34, 34, 44, 48, 53, 51, 42, 39, 37, 44, 94, 177, 180, 123, 99, 81, 66, 87, 46, 32, 40, 48, 70, 118, 100, 93, 88, 74, 62, 75, 103, 128, 127, 100, 75, 63, 50, 42, 44, 52, 45, 40, 40, 41, 51, 51, 46, 42, 59, 69, 93, 115, 111, 82, 65, 73, 77, 92, 103, 109, 104, 88, 74, 67, 39, 29, 38, 51, 56, 48, 44, 50, 60, 75, 95, 108, 109, 99, 93, 83, 118, 122, 106, 71, 48, 50, 62, 62, 106, 98, 94, 77, 53, 27, 23, 26, 18, 19, 28, 35, 38, 42, 46, 46, 53, 52, 57, 52, 39, 36, 39, 46, 39, 108, 134, 120, 105, 93, 78, 81, 37, 32, 34, 66, 90, 111, 101, 93, 68, 53, 50, 57, 85, 110, 113, 85, 57, 52, 52, 44, 44, 53, 48, 38, 35, 32, 40, 44, 42, 40, 60, 63, 87, 115, 110, 71, 55, 64, 56, 62, 79, 103, 110, 104, 91, 76, 52, 39, 45, 58, 62, 46, 38, 37, 44, 56, 71, 86, 96, 96, 101, 93, 54, 64, 62, 58, 55, 47, 43, 43, 90, 99, 102, 84, 58, 29, 25, 26, 22, 28, 44, 51, 50, 48, 53, 52, 59, 58, 59, 51, 38, 33, 40, 43, 40, 36, 41, 74, 103, 109, 81, 53, 32, 36, 32, 96, 103, 91, 93, 79, 55, 59, 84, 103, 109, 103, 85, 75, 69, 71, 61, 37, 34, 46, 46, 36, 28, 24, 32, 44, 48, 51, 60, 58, 80, 115, 110, 67, 48, 58, 62, 50, 57, 80, 102, 113, 102, 83, 66, 45, 44, 56, 64, 52, 48, 51, 56, 61, 67, 77, 87, 100, 103, 106, 104, 83, 58, 44, 47, 49, 58, 72, 60, 73, 80, 87, 75, 64, 48, 36, 37, 49, 64, 66, 60, 50, 50, 51, 64, 64, 59, 49, 38, 33, 38, 41, 54, 26, 45, 103, 115, 95, 72, 52, 31, 38, 33, 118, 106, 68, 58, 45, 49, 88, 146, 170, 152, 116, 77, 89, 107, 111, 85, 50, 41, 50, 49, 36, 25, 23, 34, 49, 57, 64, 60, 54, 74, 118, 108, 65, 40, 56, 80, 58, 46, 60, 94, 110, 100, 84, 78, 54, 48, 62, 73, 79, 79, 82, 81, 77, 76, 82, 93, 107, 112, 119, 110, 86, 69, 66, 69, 78, 73, 79, 36, 45, 63, 80, 93, 97, 69, 48, 52, 68, 81, 75, 65, 50, 46, 45, 64, 68, 60, 48, 38, 35, 33, 43, 46, 48, 123, 187, 136, 66, 63, 77, 28, 37, 35, 92, 83, 76, 54, 42, 42, 86, 150, 175, 142, 105, 110, 85, 72, 79, 79, 58, 36, 42, 45, 54, 40, 22, 24, 48, 53, 48, 60, 61, 86, 124, 96, 33, 25, 54, 44, 55, 58, 63, 78, 91, 96, 92, 63, 50, 50, 64, 63, 58, 56, 62, 98, 62, 99, 159, 189, 181, 157, 113, 124, 102, 86, 74, 82, 78, 68, 54, 37, 37, 62, 105, 106, 87, 80, 88, 48, 54, 61, 59, 61, 50, 44, 37, 34, 26, 24, 41, 49, 41, 31, 32, 48, 63, 107, 157, 155, 99, 65, 80, 62, 30, 24, 76, 75, 75, 40, 39, 48, 79, 116, 133, 117, 99, 104, 76, 58, 61, 60, 46, 37, 48, 50, 53, 37, 22, 26, 54, 65, 61, 60, 51, 75, 115, 93, 46, 50, 88, 73, 63, 51, 61, 83, 99, 96, 85, 72, 40, 45, 83, 116, 102, 69, 48, 43, 34, 65, 123, 170, 229, 226, 163, 118, 105, 89, 92, 94, 89, 71, 58, 63, 62, 64, 75, 73, 66, 78, 99, 90, 73, 59, 47, 37, 43, 50, 58, 42, 37, 42, 48, 40, 35, 51, 77, 121, 87, 83, 97, 103, 79, 69, 82, 61, 28, 22, 51, 61, 70, 31, 36, 54, 66, 72, 81, 84, 84, 99, 71, 51, 49, 46, 44, 46, 59, 54, 48, 31, 21, 31, 61, 78, 75, 69, 58, 72, 105, 88, 57, 66, 107, 98, 70, 45, 58, 85, 103, 97, 87, 76, 42, 49, 89, 116, 84, 51, 49, 44, 47, 63, 80, 130, 221, 232, 157, 93, 86, 84, 88, 98, 98, 85, 72, 69, 66, 63, 51, 36, 27, 46, 72, 108, 90, 75, 61, 48, 54, 57, 67, 46, 49, 59, 53, 32, 28, 69, 120, 176, 112, 75, 71, 80, 71, 66, 71, 71, 51, 39, 34, 49, 59, 39, 40, 51, 52, 49, 52, 63, 68, 84, 70, 58, 54, 50, 52, 58, 61, 54, 39, 21, 22, 40, 68, 82, 85, 85, 77, 83, 102, 87, 54, 54, 88, 90, 68, 46, 59, 78, 90, 95, 102, 83, 57, 52, 74, 66, 37, 30, 64, 70, 60, 68, 63, 89, 149, 170, 117, 60, 56, 58, 61, 78, 87, 90, 88, 66, 66, 61, 55, 38, 25, 27, 34, 80, 87, 93, 93, 90, 75, 58, 53, 46, 51, 57, 51, 33, 30, 66, 112, 153, 118, 96, 106, 105, 79, 56, 47, 67, 63, 50, 28, 42, 48, 62, 52, 43, 43, 48, 53, 55, 51, 64, 67, 66, 59, 51, 52, 53, 49, 43, 29, 19, 30, 57, 74, 84, 84, 83, 79, 89, 103, 88, 51, 43, 57, 64, 54, 53, 69, 72, 74, 89, 115, 104, 65, 42, 62, 78, 76, 61, 64, 64, 36, 46, 70, 61, 82, 105, 104, 57, 48, 32, 37, 42, 55, 67, 78, 78, 73, 60, 57, 57, 58, 53, 42, 53, 67, 85, 92, 96, 73, 54, 37, 49, 43, 38, 42, 38, 32, 43, 63, 86, 97, 112, 128, 111, 72, 48, 40, 65, 58, 47, 31, 41, 44, 77, 62, 43, 45, 57, 64, 57, 43, 50, 62, 63, 51, 41, 37, 36, 31, 34, 24, 23, 51, 80, 87, 85, 78, 69, 69, 87, 102, 89, 57, 44, 42, 48, 41, 59, 86, 79, 67, 82, 113, 121, 73, 34, 62, 141, 177, 139, 61, 80, 36, 43, 66, 48, 49, 72, 86, 79, 62, 40, 27, 26, 30, 43, 53, 93, 88, 73, 53, 52, 69, 78, 73, 54, 54, 61, 61, 59, 53, 48, 37, 56, 34, 22, 35, 38, 32, 25, 25, 38, 68, 99, 95, 70, 52, 49, 64, 96, 83, 56, 37, 47, 49, 78, 65, 53, 53, 62, 65, 60, 51, 58, 73, 66, 42, 24, 27, 30, 26, 33, 23, 35, 79, 110, 105, 85, 74, 71, 73, 88, 96, 86, 61, 45, 38, 51, 41, 61, 101, 103, 77, 76, 96, 111, 93, 56, 56, 135, 208, 186, 99, 104, 60, 52, 42, 47, 78, 98, 77, 89, 76, 62, 37, 34, 36, 51, 56, 123, 144, 147, 109, 74, 69, 78, 83, 69, 63, 58, 58, 45, 48, 45, 45, 58, 33, 19, 31, 35, 27, 24, 25, 25, 52, 71, 55, 37, 41, 66, 96, 117, 104, 73, 40, 51, 56, 71, 65, 64, 62, 62, 59, 63, 64, 75, 90, 73, 40, 18, 27, 33, 32, 39, 26, 46, 98, 130, 120, 84, 71, 86, 93, 94, 92, 81, 59, 38, 34, 61, 48, 61, 106, 121, 88, 72, 77, 86, 119, 90, 44, 71, 165, 187, 151, 88, 61, 38, 23, 50, 138, 163, 104, 83, 79, 70, 62, 54, 59, 72, 81, 167, 226, 239, 198, 132, 89, 77, 75, 78, 81, 73, 78, 67, 60, 44, 48, 54, 38, 26, 32, 31, 24, 29, 44, 26, 49, 54, 41, 33, 42, 85, 109, 102, 91, 70, 16, 31, 52, 65, 76, 85, 70, 52, 50, 64, 85, 85, 118, 78, 37, 48, 24, 5, 43, 34, 47, 35, 39, 75, 82, 85, 100, 95, 97, 122, 108, 59, 49, 51, 35, 38, 49, 75, 107, 120, 97, 71, 71, 89, 109, 94, 55, 42, 91, 155, 202, 90, 55, 37, 52, 79, 89, 111, 124, 123, 109, 92, 85, 77, 66, 78, 101, 119, 133, 150, 124, 92, 75, 90, 110, 48, 67, 85, 102, 93, 79, 57, 41, 40, 40, 35, 27, 29, 27, 25, 28, 19, 50, 51, 32, 24, 55, 67, 54, 67, 74, 68, 29, 64, 85, 76, 80, 75, 73, 71, 68, 79, 76, 92, 194, 166, 57, 39, 54, 35, 21, 23, 31, 22, 40, 95, 104, 83, 79, 78, 71, 97, 97, 53, 38, 51, 41, 73, 72, 77, 89, 103, 101, 81, 61, 84, 99, 102, 64, 42, 56, 91, 111, 75, 52, 35, 44, 55, 61, 71, 91, 175, 144, 102, 73, 69, 77, 74, 66, 54, 64, 69, 75, 68, 76, 95, 105, 61, 86, 118, 130, 111, 88, 78, 77, 40, 37, 33, 38, 41, 34, 24, 15, 50, 64, 58, 34, 50, 76, 70, 35, 36, 56, 70, 62, 109, 139, 81, 79, 71, 80, 85, 85, 72, 64, 55, 171, 179, 78, 39, 52, 42, 34, 34, 34, 23, 62, 134, 146, 100, 75, 76, 53, 76, 94, 54, 29, 51, 61, 141, 136, 103, 79, 87, 104, 79, 38, 59, 88, 107, 82, 57, 38, 37, 35, 43, 35, 32, 38, 35, 35, 43, 66, 158, 131, 96, 76, 108, 140, 114, 60, 57, 54, 42, 40, 50, 59, 70, 75, 66, 104, 141, 147, 125, 102, 96, 108, 62, 51, 37, 35, 35, 35, 28, 22, 51, 50, 32, 27, 63, 95, 79, 34, 20, 41, 62, 81, 126, 162, 76, 80, 85, 92, 84, 79, 57, 53, 49, 105, 138, 117, 71, 26, 17, 47, 34, 37, 28, 68, 138, 152, 108, 90, 92, 60, 78, 98, 57, 29, 52, 72, 155, 151, 113, 80, 90, 123, 96, 38, 42, 72, 103, 107, 81, 55, 35, 30, 18, 23, 40, 44, 35, 33, 49, 69, 68, 66, 62, 85, 164, 224, 181, 90, 90, 83, 66, 51, 54, 55, 64, 73, 64, 93, 122, 127, 118, 102, 94, 100, 90, 81, 61, 33, 23, 26, 36, 46, 55, 43, 18, 17, 48, 81, 77, 49, 48, 50, 51, 64, 91, 115, 79, 97, 115, 112, 88, 62, 61, 49, 89, 115, 152, 149, 102, 59, 45, 30, 13, 30, 23, 48, 98, 109, 93, 101, 107, 76, 86, 95, 47, 30, 55, 67, 103, 107, 97, 81, 95, 131, 110, 52, 37, 56, 93, 116, 95, 72, 55, 56, 24, 43, 65, 58, 45, 42, 57, 72, 24, 28, 30, 65, 155, 223, 183, 95, 70, 79, 72, 71, 59, 67, 90, 107, 62, 59, 73, 82, 88, 85, 74, 64, 82, 95, 92, 63, 41, 27, 40, 58, 69, 54, 37, 18, 40, 71, 102, 107, 108, 75, 54, 35, 49, 55, 90, 112, 129, 122, 97, 66, 62, 61, 76, 129, 168, 123, 89, 134, 137, 44, 23, 44, 31, 38, 73, 84, 88, 109, 103, 85, 93, 78, 33, 38, 62, 68, 75, 96, 99, 82, 79, 82, 65, 33, 35, 47, 82, 104, 97, 72, 67, 69, 51, 76, 86, 64, 52, 49, 52, 59, 54, 41, 22, 28, 81, 131, 112, 65, 58, 68, 71, 65, 57, 65, 84, 107, 54, 40, 38, 42, 55, 59, 53, 38, 49, 78, 98, 99, 78, 54, 48, 54, 40, 37, 31, 33, 52, 99, 163, 200, 148, 99, 59, 44, 58, 61, 89, 101, 110, 110, 94, 78, 57, 62, 65, 98, 137, 92, 65, 146, 184, 106, 45, 58, 39, 35, 71, 81, 85, 107, 87, 93, 100, 66, 32, 62, 96, 90, 72, 102, 113, 95, 64, 32, 23, 25, 31, 41, 74, 97, 99, 77, 75, 82, 58, 80, 80, 58, 48, 52, 51, 45, 67, 42, 25, 20, 33, 49, 50, 49, 71, 67, 68, 60, 65, 71, 75, 85, 52, 45, 34, 30, 34, 40, 43, 42, 40, 50, 67, 86, 89, 87, 73, 66, 28, 28, 20, 34, 46, 95, 153, 200, 145, 106, 68, 74, 94, 108, 81, 78, 83, 91, 87, 81, 61, 49, 116, 81, 107, 105, 64, 97, 157, 152, 40, 51, 30, 22, 61, 69, 71, 92, 70, 97, 110, 63, 36, 91, 142, 112, 52, 71, 107, 107, 63, 30, 30, 56, 36, 35, 68, 105, 104, 89, 85, 103, 48, 61, 63, 50, 42, 56, 60, 44, 37, 20, 27, 37, 37, 31, 38, 63, 63, 55, 49, 65, 86, 102, 100, 91, 61, 54, 44, 35, 28, 30, 42, 59, 57, 35, 27, 44, 75, 103, 100, 88, 77, 58, 34, 24, 24, 37, 71, 101, 127, 100, 81, 143, 105, 134, 109, 87, 101, 112, 63, 52, 65, 65, 168, 152, 135, 109, 88, 88, 113, 152, 58, 37, 21, 36, 57, 79, 74, 61, 80, 79, 98, 89, 42, 59, 132, 180, 108, 90, 88, 97, 76, 34, 23, 43, 19, 29, 48, 71, 88, 98, 92, 88, 61, 42, 48, 73, 78, 53, 38, 30, 31, 32, 27, 21, 29, 41, 54, 67, 67, 66, 74, 70, 58, 61, 76, 99, 104, 89, 112, 126, 75, 46, 42, 37, 50, 58, 56, 48, 37, 49, 81, 107, 90, 80, 62, 40, 33, 37, 32, 31, 59, 55, 55, 177, 108, 89, 104, 88, 114, 128, 92, 66, 72, 59, 191, 150, 95, 56, 71, 107, 144, 160, 52, 35, 20, 28, 58, 80, 81, 78, 96, 108, 112, 78, 46, 90, 153, 159, 107, 99, 94, 86, 66, 38, 29, 36, 20, 25, 35, 58, 78, 87, 80, 71, 77, 54, 52, 68, 73, 53, 37, 28, 26, 22, 20, 31, 48, 53, 49, 43, 35, 45, 54, 44, 34, 35, 55, 69, 123, 123, 141, 184, 179, 103, 46, 31, 37, 59, 75, 76, 61, 58, 67, 79, 88, 95, 89, 73, 53, 41, 38, 29, 29, 29, 31, 159, 97, 81, 102, 97, 137, 157, 125, 92, 87, 68, 131, 129, 116, 84, 90, 114, 119, 106, 50, 32, 24, 28, 56, 73, 76, 73, 88, 104, 105, 64, 48, 112, 159, 132, 104, 113, 102, 73, 52, 45, 38, 29, 25, 19, 18, 38, 63, 74, 65, 52, 85, 71, 65, 68, 65, 45, 32, 31, 35, 23, 18, 35, 58, 71, 61, 41, 19, 29, 33, 26, 19, 20, 31, 51, 80, 101, 105, 158, 214, 141, 48, 49, 55, 55, 54, 36, 29, 27, 42, 55, 73, 88, 104, 113, 93, 73, 72, 73, 107, 111, 88, 119, 83, 105, 90, 87, 127, 152, 115, 88, 85, 70, 68, 116, 147, 119, 89, 86, 70, 46, 47, 37, 32, 40, 59, 65, 64, 51, 63, 62, 86, 81, 62, 97, 129, 114, 104, 125, 110, 65, 44, 51, 46, 26, 32, 22, 10, 26, 51, 63, 57, 43, 68, 78, 86, 81, 55, 30, 23, 27, 38, 31, 23, 35, 46, 60, 58, 52, 28, 28, 29, 28, 23, 26, 29, 48, 74, 125, 95, 91, 166, 142, 61, 86, 88, 71, 59, 51, 55, 59, 58, 53, 54, 58, 72, 102, 107, 104, 98, 114, 194, 200, 156, 152, 100, 99, 77, 63, 93, 114, 79, 62, 74, 63, 82, 102, 99, 82, 75, 80, 63, 30, 34, 38, 33, 52, 67, 74, 69, 57, 75, 49, 89, 134, 105, 72, 88, 108, 106, 125, 108, 68, 46, 54, 50, 32, 38, 27, 14, 24, 42, 58, 54, 50, 42, 75, 102, 92, 50, 25, 23, 24, 41, 55, 70, 78, 62, 45, 33, 38, 32, 31, 32, 35, 29, 34, 34, 39, 119, 207, 152, 75, 122, 132, 91, 104, 135, 128, 120, 146, 167, 178, 150, 124, 92, 66, 46, 69, 101, 105, 96, 98, 138, 162, 145, 219, 126, 67, 83, 65, 84, 93, 70, 62, 80, 84, 84, 54, 41, 75, 133, 134, 74, 19, 27, 25, 37, 54, 76, 95, 101, 105, 112, 69, 107, 175, 144, 68, 69, 103, 106, 114, 101, 78, 58, 52, 48, 42, 40, 33, 22, 25, 33, 49, 58, 64, 35, 69, 95, 85, 55, 41, 42, 41, 59, 101, 144, 153, 123, 72, 32, 21, 29, 31, 33, 33, 31, 32, 34, 32, 78, 160, 134, 62, 79, 105, 102, 96, 203, 178, 146, 148, 177, 212, 234, 234, 186, 120, 73, 79, 103, 99, 87, 80, 59, 98, 112, 198, 113, 63, 87, 70, 85, 92, 68, 66, 89, 102, 72, 31, 43, 111, 170, 151, 90, 52, 43, 31, 42, 50, 78, 111, 135, 156, 113, 69, 96, 154, 136, 85, 77, 84, 104, 97, 93, 92, 73, 50, 42, 51, 35, 35, 30, 26, 24, 38, 60, 78, 59, 68, 69, 58, 68, 78, 75, 59, 70, 109, 148, 161, 138, 94, 49, 24, 32, 32, 34, 31, 35, 33, 31, 32, 21, 43, 50, 40, 40, 62, 83, 96, 190, 165, 132, 97, 99, 129, 191, 231, 213, 134, 86, 86, 96, 81, 76, 83, 70, 84, 90, 122, 77, 78, 69, 52, 68, 82, 51, 49, 80, 86, 93, 63, 64, 112, 122, 100, 117, 153, 74, 65, 46, 49, 74, 113, 154, 179, 84, 43, 62, 106, 103, 102, 90, 64, 103, 87, 88, 102, 84, 49, 38, 57, 31, 34, 34, 25, 16, 29, 59, 85, 87, 69, 43, 33, 78, 114, 96, 61, 58, 73, 85, 101, 99, 78, 55, 34, 41, 34, 38, 33, 43, 39, 36, 28, 50, 19, 20, 50, 39, 33, 66, 111, 94, 119, 141, 123, 73, 56, 80, 111, 155, 101, 54, 53, 67, 52, 57, 77, 106, 71, 57, 66, 76, 102, 87, 75, 60, 54, 44, 53, 59, 71, 85, 80, 67, 89, 113, 106, 127, 196, 197, 78, 26, 65, 102, 84, 66, 76, 44, 61, 67, 96, 162, 159, 101, 84, 107, 101, 97, 79, 49, 25, 36, 54, 63, 45, 25, 19, 18, 38, 54, 71, 70, 60, 41, 42, 120, 198, 172, 102, 80, 79, 51, 34, 73, 108, 87, 31, 33, 28, 34, 33, 32, 35, 30, 30, 23, 41, 55, 49, 34, 37, 72, 102, 177, 136, 94, 73, 82, 89, 72, 46, 76, 96, 92, 75, 87, 112, 95, 54, 102, 55, 77, 57, 71, 77, 127, 115, 92, 70, 56, 45, 39, 40, 63, 94, 104, 115, 114, 84, 103, 179, 218, 112, 42, 61, 87, 60, 44, 62, 45, 70, 84, 130, 176, 167, 119, 107, 114, 102, 99, 98, 78, 44, 25, 25, 50, 52, 43, 31, 26, 37, 59, 84, 88, 78, 48, 46, 121, 194, 175, 106, 80, 88, 81, 74, 98, 129, 107, 59, 33, 35, 35, 37, 23, 21, 21, 32, 44, 61, 63, 49, 34, 34, 53, 71, 118, 89, 71, 82, 108, 119, 94, 76, 68, 88, 92, 89, 114, 166, 182, 159, 119, 52, 73, 60, 55, 57, 80, 76, 63, 48, 44, 38, 37, 37, 52, 98, 121, 121, 103, 62, 87, 170, 239, 153, 72, 71, 76, 46, 26, 44, 49, 68, 85, 139, 173, 145, 101, 104, 103, 97, 100, 114, 106, 68, 25, 9, 36, 52, 57, 47, 37, 41, 69, 93, 79, 67, 37, 34, 92, 156, 150, 102, 98, 122, 129, 120, 112, 123, 107, 78, 54, 61, 62, 62, 46, 42, 54, 61, 67, 63, 57, 44, 29, 26, 29, 42, 62, 54, 62, 93, 127, 138, 111, 93, 68, 86, 92, 88, 114, 172, 212, 226, 168, 102, 89, 76, 55, 53, 25, 26, 24, 26, 29, 36, 41, 48, 53, 86, 94, 99, 107, 89, 109, 170, 238, 165, 96, 90, 79, 46, 26, 40, 55, 62, 73, 119, 168, 132, 87, 89, 86, 95, 103, 121, 108, 71, 35, 19, 34, 40, 51, 54, 54, 58, 75, 83, 58, 51, 29, 26, 63, 120, 122, 95, 127, 156, 158, 130, 95, 80, 70, 60, 66, 62, 66, 64, 75, 78, 88, 83, 84, 69, 59, 52, 46, 45, 42, 46, 59, 61, 75, 99, 119, 122, 105, 79, 72, 91, 89, 71, 74, 104, 149, 166, 144, 90, 59, 89, 77, 63, 41, 37, 33, 36, 36, 37, 37, 44, 45, 64, 66, 99, 158, 163, 146, 155, 193, 133, 93, 97, 81, 44, 28, 41, 38, 53, 58, 93, 163, 160, 121, 112, 93, 101, 108, 112, 81, 42, 29, 32, 36, 28, 39, 55, 72, 80, 72, 63, 49, 53, 37, 32, 61, 104, 108, 90, 133, 147, 143, 105, 63, 41, 32, 30, 44, 28, 29, 38, 73, 87, 77, 67, 80, 78, 68, 66, 71, 77, 75, 68, 79, 78, 80, 89, 97, 93, 86, 69, 67, 86, 87, 62, 47, 51, 73, 84, 132, 121, 105, 99, 89, 78, 53, 45, 45, 35, 37, 38, 43, 48, 28, 48, 62, 120, 200, 206, 156, 124, 121, 86, 70, 85, 73, 40, 25, 36, 16, 52, 51, 63, 127, 163, 150, 136, 107, 103, 101, 87, 46, 13, 14, 31, 34, 29, 37, 62, 88, 92, 71, 49, 36, 49, 52, 44, 54, 72, 79, 66, 97, 94, 92, 68, 44, 32, 24, 25, 30, 19, 20, 38, 68, 78, 61, 45, 50, 49, 45, 49, 56, 62, 67, 62, 71, 67, 68, 68, 70, 69, 70, 72, 55, 72, 83, 71, 58, 52, 52, 73, 153, 192, 207, 100, 92, 82, 52, 42, 38, 28, 31, 38, 51, 54, 25, 48, 66, 104, 164, 166, 122, 103, 77, 61, 59, 73, 63, 37, 27, 38, 46, 115, 103, 56, 79, 125, 131, 119, 101, 83, 68, 54, 32, 9, 15, 33, 26, 38, 58, 79, 98, 92, 72, 49, 37, 52, 75, 64, 52, 45, 45, 38, 53, 44, 44, 48, 50, 45, 36, 30, 35, 44, 61, 70, 66, 57, 46, 35, 24, 24, 26, 27, 29, 28, 36, 41, 42, 41, 49, 44, 46, 48, 54, 63, 59, 66, 78, 81, 72, 66, 68, 77, 103, 150, 164, 89, 99, 74, 83, 70, 52, 43, 31, 36, 40, 46, 29, 64, 62, 62, 95, 97, 86, 104, 72, 62, 63, 75, 62, 40, 35, 54, 119, 209, 189, 78, 58, 95, 104, 87, 81, 55, 33, 31, 32, 24, 31, 41, 22, 47, 76, 95, 102, 88, 70, 58, 62, 80, 105, 92, 72, 52, 36, 28, 32, 25, 22, 44, 67, 62, 47, 25, 36, 64, 95, 89, 58, 28, 17, 24, 26, 38, 40, 32, 27, 26, 27, 35, 23, 26, 38, 29, 32, 31, 44, 43, 73, 71, 77, 84, 73, 64, 62, 63, 59, 103, 75, 77, 84, 99, 87, 98, 98, 86, 79, 75, 54, 32, 43, 35, 50, 75, 86, 76, 75, 89, 80, 71, 45, 53, 88, 71, 38, 47, 132, 172, 157, 85, 62, 102, 114, 85, 73, 44, 23, 27, 37, 34, 26, 24, 43, 52, 75, 94, 88, 77, 89, 114, 97, 112, 115, 97, 63, 33, 25, 32, 19, 26, 32, 33, 29, 32, 37, 46, 100, 84, 60, 38, 23, 19, 24, 28, 31, 29, 28, 27, 26, 29, 32, 34, 27, 28, 25, 26, 27, 28, 31, 32, 35, 46, 64, 73, 73, 71, 72, 72, 59, 70, 71
};

GlGrowthPattern* gl_new_hair_cap_moss_pattern()
{
	GlGrowthPattern*	pattern = new GlGrowthPattern(gHairCapMossData, gHairCapMossW, gHairCapMossH);
	if (!pattern) return 0;
	if (pattern->InitCheck() == B_OK) return pattern;
	delete pattern;
	return 0;
}
